#ifndef _DLCANNYEDGEDETECT_H_
#define _DLCANNYEDGEDETECT_H_

#include "DLImage.h"
#include "DLBitsPerPixelConverter.h"
#include "DLConst.h"
#include "DLTypes.h"

#include <math.h>
#include <stdio.h>

using namespace std;

/// scaling factors for magnitude and orientation to fit within an unsigned char
#define MAGNITUDE_SCALE 7.0
#define ORIENTATION_SCALE 40.0
#define MAX_RECURSION 200

/// standard deviation of the Gaussian distribution function
#define STANDARD_DEVIATION 1.0
#define PI 3.1415927

/**
* \brief The DLCannyEdgeDetect class is a complete implementation of the well-known Canny edge detector.
* It computes both the detected edge magnitude and orientation information. 
* 
* Two threshold parameters highT and lowT (i.e. highT > lowT) in the range of [0, 255] control
* the quality of computed edges as they are used globally
* at the edge tracking step of the Canny edge detector. Edge tracking starts at a point that
* has edge magnitude higher than highT and then continues outwards from that starting point
* until it reaches a point that has an edge magnitude lower than lowT. This hysteresis reduces
* the chance that noisy edges are broken up into multiple edge fragments.
*
* Users can obtain these information from its member functions dlGetMagnitudeImage() and
* dlGetOrientationImage(), respectively. Both the edge magnitude and orientation are encoded
* within the intensity range of grayscale image, i.e. [0, 255].
*
* Reference:<br>
* J. Canny, "A Computational Approach to Edge Detection," <i>IEEE Trans. 
* Pattern Analysis and Machine Intelligence</i>, vol. 8, no. 6, pp. 679--698, 1986.
*/

class DLCannyEdgeDetect
{
public:

	/**
	* Default DLCannyEdgeDetect Constructor
	*/
	DLCannyEdgeDetect();

	/**
	* DLCannyEdgeDetect Constructor
	*
	* \param image pointer to the source DLImage
	* \param lowT lower threshold used in edge tracking, setting lowT too high causes noisy edges to break up
	* \param highT higher threshold used in edge tracking, setting highT too low increases the number of spurious and undesirable edge fragments
	*
	*/
	DLCannyEdgeDetect(const DLImage& image, DL_BYTE lowT=1, DL_BYTE highT=255);

	/**
	* Default DLCannyEdgeDetect Destructor
	*/
	virtual ~DLCannyEdgeDetect();

	/**
	* Return magnitude of detected edges (edge map) as a grayscale image
	* \return a grayscale DLImage that represents the magnitude of detected edges
	*/
	DLImage dlGetMagnitudeImage();

	/**
	* Return orientation of detected edges (scaled within an unsigned char 0-255) as a grayscale image
	* \return a grayscale DLImage that encodes the orientation information of detected edges
	*/
	DLImage dlGetOrientationImage();

private:

	/// Height of source image
	int rows;

	/// Width of source image
	int cols;

	int recCounter;

	/// pointer to a copy of the imageData (DL_BYTE array) of input image
	/// also used at the end for storing final edge magnitude after threshold tracking
	DL_BYTE *data;
    
	/// pointer to DL_BYTE array that stores edge magnitude before threshold tracking (i.e. intermediate edge magnitude)
    DL_BYTE *magnitude;

	/// pointer to DL_BYTE array that stores edge orientation
	DL_BYTE *orientation;

	/**
	* Core function for the canny edge detector
	* \param s the standard deviation of the Gaussian distribution in float
	* \param lowT lower threshold used in edge tracking, setting lowT too high causes noisy edges to break up
	* \param highT higher threshold used in edge tracking, setting highT too low increases the number of spurious and undesirable edge fragments
	*/
	void dlDetectEdge(float s, DL_BYTE lowT, DL_BYTE highT);

	/**
	* Track the edges using two specified thresholds - lowT and highT, starting at a point that has
	* edge magnitudes higher than highT and then continues outwards from that starting point until
	* it reaches a point that has an edge magnitude lower than lowT. 
	* This hysteresis reduces the possibilities that noisy edges are broken up into multiple edge fragments.
	* \param lowT lower threshold used in edge tracking, setting lowT too high causes noisy edges to break up
	* \param highT higher threshold used in edge tracking, setting highT too low increases the number of spurious and undesirable edge fragments
	*/
	void trackEdgeThreshold(DL_BYTE lowT, DL_BYTE highT);

	/**
	* Follow a connected edge recursively, starting from a strong edge point.
	* This is a private member function used only in the edge tracking hysteresis.
	* \param i int row number of the current edge point
	* \param j int col number of the current edge point
	* \param lowT DL_BYTE lower threshold used in edge tracking, setting lowT too high causes noisy edges to break up
	* \return bool of 1 if edge reaches the end successfully, 0 if error occurs
	*/
    bool followEdge(int i, int j, DL_BYTE lowT);

	/**
	*  Compute the length of the hypotenuse of a right-angled triangle with sides of length x and y
	* \param x length of a side of the right-angled triangle
	* \param y length of the other side of the right-angled triangle
	* \return the length of the hypotenuse of a right-angled triangle with sides of length x and y
	*/
	float hypotenuse(float x, float y)
	{
		if (x==0.0 && y==0.0) 
			return 0.0;
		else
			return (float)(hypot(x,y));
	};

	/**
	* Gaussian (normal) distribution function with zero mean and the specified standard deviation
	* \param x input parameter
	* \param s standard deviation of the Gaussian distribution
	* \return float Gaussian probability distribution of at the input value of parameter x
	*/	
	float gaussianDistribution(float x, float s) { return exp((-x*x)/(2*s*s)); };
	
};//end class

#endif //_DLCANNYEDGEDETECT_H_
