#ifndef _DLTEST_H
#define _DLTEST_H

#include <string>
using namespace std;

/// Displayed messages
const static char* RESULTS[] = {{"Failed"}, {"Successful"}};


/**
* \brief The DLTest class provides an template for an individual driver application. 
*
* Developers can use the DLTest template to create a customized driver application using DOCLIB and its add-on
* modules by defining a derived class from DLTest.  
* Two virtual functions need to be customized in a derived class of DLTest -- go() and showHelp(). 
* Function go() should include the procedures of the application, including how to parse the set of command line arguments
* and what steps to execute (e.g. looping through a set of documents specified by the command line user).
* Function showHelp() displays the command-line help information to the end user, so that a new user
* is aware of what to expect when running the application and the command line syntax to run the application. 
* 
*/
class DLTest
{
public:

	/**
    * Default DLTest Constructor. Initialize the list of command line arguments
	* \param argc An integer that contains the count of arguments that follow in argv. 
	* The argc parameter is always greater than or equal to 1. 
	* \param argv An array of null-terminated strings representing command-line arguments
	* entered by the user of the program. By convention, argv[0] is the command with which the
	* program is invoked, argv[1] is the first command-line argument, and so on, until argv[argc],
	* which is always NULL.
	* \param name A string given to the test.
    */
	DLTest( char* name, int argc, char** argv);

	/**
    * Default DLTest Destructor
    */
	virtual ~DLTest(void);

	/**
    * Return the name of the test
	* \return the name of the test 
    */
	string getTestName() {return  static_cast<string>(testName);};

	/**
    * Return the final status of the test
	* \return the final status of the test
    */
	string getResult() {return static_cast<string>(RESULTS[result]);};

	/**
    * Append new information in string to the test log
    */
	void appendToTestLog(string newInfo);

	/**
    * Return the complete test log string
	* \return the complete test log string
    */
	string getTestLog() {return testLog;};

	/**
    * Developers need to write the set of actions in this member function of their derived class as this is a virtual function
    */
	virtual int go()=0;

	/**
    * Developers need to define command line help information displayed to users in this member function of their derived class
	* as this is a virtual function
    */
	virtual void showHelp()=0;

protected:
	
	/// The number of arguments that follow in dlArgv
	int dlArgc;

	/// The array of null-terminated strings representing command-line arguments
	char** dlArgv;

	/// Name of the test
	char* testName;

	/// The test log information
	string testLog;

	/// Current status of the test
	int result;
};


#endif //_DLTEST_H
