#ifndef _DLLOGODETECT_H_
#define _DLLOGODETECT_H_

#include "DLLogoDetect.h"
#include "DLImage.h"
#include "DLComponent.h"
#include "DLConnectedComponents.h"
#include "DLBitsPerPixelConverter.h"
#include "DLZone.h"

#include <string>

/// Default logo detection threshold. A reasonable range of detection threshold is above 600
/// Higher detection threshold gives better precision, but may lower the recall.
const double MIN_SCORE = 700.0;

/**
 * \class DLLogoDetect
 *
 * \brief DLLogoDetect provides the implementation of the multi-scale logo detection and extraction approach
 * by Zhu and Doermann at ICDAR 2007. It returns the most likely logo candidate on each page whose computed scores are
 * above the specified detection threshold. A higher detection threshold leads to better precision (i.e. higher
 * chance of being really a logo), but may lower the recall. A default detection threshold is used if that
 * parameter is ignored by the user.
 *
 * The idea of DLLogoDetect is to robustly classifies and precisely localizes logos using a boosting strategy
 * across multiple image scales. At a coarse scale, a trained Fisher classifier performs an
 * initial classification using features from document context and connected components. Each
 * logo candidate region is further classified at successively finer image scales by a cascade
 * of classifiers, which allows false alarms to be quickly discarded and the detected
 * region to be refined.
 * 
 * We assume that the logo in each document appears on the top one third of the document.
 * If more than one logos are present on a document, the one with highest computed score is selected.
 *
 */

 /**
 *  \brief struct for storing a logo candidate and its associated information
 */
struct DLLogoCandidate
{
	/// The spatial region correponding to the logo candidate
	DLZone logoRegion;
	/// Source image file name
	string sourceImageName;
	/// Computed score of the logo candidate region
	unsigned long scores;
	/// String that stores any necessary type information for later processing needs
	string type;
};


class DLLogoDetect
{
public:

	/**
	* Default DLLogoDetect Constructor. Initialize logo detector using the file name specified
	* \param fname file name of the source image
	* \param threshold detection threshold. A reasonable range of detection threshold is above 600. 
	* Higher detection threshold gives better precision, but may lower the recall. 
	* 
	*/
	DLLogoDetect(const char *fname, double threshold=MIN_SCORE);

	/**
	* Default DLLogoDetect Constructorl Initialize logo detector using the supplied DLImage
	* \param srcImage source image to process
	* \param fname file name of the source image if available
	* \param threshold detection threshold. A reasonable range of detection threshold is above 600. 
	* Higher detection threshold gives better precision, but may lower the recall. 
	* 
	*/
	DLLogoDetect(const DLImage &srcImage, const char* fname, double threshold=MIN_SCORE);

	/**
	* Return the detected logo
	* \return the detected logo of DLLogoCandidate type. The list is empty is none logo is detected
	*/
	list <DLLogoCandidate> dlGetDetectedLogos() { return logoList; };

	/**
	* Check whether any logo is detected on the document page
	* \return true if at least one logo is detected
	*/
	bool dlGotLogo() { return !logoList.empty(); };


private:

	/**
    * Run the logo detector on the input document image
	* \param decisionThreshold the detection threshold of the Fisher classifier. 
	* A reasonable range of detection threshold is above 600.
	* Higher detection threshold gives better precision, but may lower the recall. 
	*/
    void dlDetectLogo(double threshold);

     /**
     * Compute the projection of the feature set onto the direction given by the Fisher classifier
	 * \param ccListi the connected component to analyze
     * \return the projection value in int
     */
     int dlFisherClassify(const DLComponent & ccListi);
     
     /**
     * Compute the spatial density
	 * \param ccListi the connected component to analyze
     * \return the spatial density in double
     */
	 double dlComputeSpatialDensity(const DLComponent & ccListi);

	 /**
     * Initiate the cascade of classifiers
	 * \param ccListi the connected component to analyze
	 * \param origin A two-element integer array storing the location of the upper left corner of the logo region in the coarsest scale.
     * origin[0] is its x coordinate, and origin[1] is its y coordinate.
	 * \param size A two-element integer array storing the dimensions of the logo candidate region in the coarsest scale.
     * size[0] is width, and size[1] is its height.
	 * \return true if successful, false if it is classified as a false alarm
     */
	 bool dlCascadeClassify(const DLComponent ccListi, int *origin, int *size);

	 /**
     * Run the cascade of classifiers on successively finer scale levels and refine the logo region initially identified
	 * \param regOriginX X coordinate of the logo region origin initially identified
	 * \param regOriginY Y coordinate of the logo region origin initially identified
	 * \param regWidth Width of the logo region initially identified
	 * \param regHeight Height of the logo region initially identified
     * \return true if successful, false if it is classified as a false alarm by a classifier
     */	 
	 bool dlMultiScaleDetect(int regOriginX, int regOriginY, int regWidth, int regHeight);

	 /// File name of the input image
	 string sourceImageName;

	 /// Input image to run the logo detector on
	 DLImage sourceImage;

	 /// Asscociated information needed for the logo detector
	 int normalizedWidth;
	 int grayThreshold;
	 int width, height;

	 /// Horizontal resolution of source image
	 double hRes;
	 /// Vertical resolution of source image
	 double vRes;

	 int i1, i2, i3, i4, i5, i6, i7, i8, i9;
	 float f1, f2;

	 /// Detected logo candidates
	list <DLLogoCandidate> logoList;

};//end class

#endif //_DLLOGODETECT_H_
