#include "DLSignatureDetectBatchTest.h"
#include "DLSignatureDetect.h"
#include "DLImage.h"
#include "DLDocument.h"
#include "DLPage.h"
#include "DLZone.h"
#include "DLTestManager.h"
#include "DLConst.h"
#include "DLxml.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

/// Max length in input file name at the command line
#define MAX_LENGTH 260

bool compScores(const DLSignatureCandidate &x, const DLSignatureCandidate &y)
{
	return (x.scores > y.scores ? true : false);
}


DLSignatureDetectBatchTest::DLSignatureDetectBatchTest(char* name, int argc, char** argv) : DLTest(name, argc, argv)
{
	testName = name;
	dlArgc = argc;
	dlArgv = argv;
}

DLSignatureDetectBatchTest::~DLSignatureDetectBatchTest(void)
{

}

void DLSignatureDetectBatchTest::showHelp()
{
	/* Show command line help information */
	printf( "SignatureDetect [-x] [-e] [-p ParameterSet] [-l] filename\n" );
	printf( "     -x  ---  Write detection results into an xml file for each input document.\n" );
	printf( "              It creates a new xml file if no associated xml file exists.\n" );
	printf( "     -e  ---  Erase all previous signature detection results from xml files.\n" );
	printf( "              Write new results to xml file.\n" );
	printf( "     -p  ---  Specify parameter set for signature detection.\n" );
	printf( "              1: Machine printed English,  2: Handwritten Arabic.\n" );
	printf( "     -l  ---  File containing the list of input images to execute\n" );
	printf( "     -h  ---  Show help at command line\n" );
}

int DLSignatureDetectBatchTest::go()
{
	char inputFile[MAX_LENGTH];
	char logFile[MAX_LENGTH];
	bool bFileList = false;
	bool bOutputXML = false;
	bool bErasePrevious = false;
	bool bPlotSaliency = false;
	int parameterSet = 1;

	if(dlArgc == 1)
	{
		showHelp();
		return 0;
	}

	for(int i=1; i<dlArgc; i++)
	{
		if(dlArgv[i][0] != '-') break;
		++i;
		switch(tolower(dlArgv[i-1][1]))
		{
			case 'x':
				bOutputXML = true;
				i--;
				break;
			case 'e':
				bErasePrevious = true;
				i--;
				break;
			case 'p':
				parameterSet = atoi( dlArgv[i] );
				if (parameterSet <= 0 || parameterSet > 2)
				{
					printf( "Invalid parameter set!\n" );
					showHelp();
					return -1;
				}
				break;
			case 'o':
				strcpy( logFile, dlArgv[i] );
				break;
			case 'l':
				bFileList = true;
				i--;
				break;
			case '?':
			case 'h':
				showHelp();
				return 0;
			default:
				printf( "Unable to interprete your command!\n" );
				showHelp();
				return -1;
		}
	}

	/* Show commmand line help panel if incorrect arguments are supplied */
	if( i != dlArgc-1 )
	{
		showHelp();
		return -1;
	}

	/* Grab the input file name specified at the command line */
	strcpy( inputFile, dlArgv[i] );

	vector <string> imageFileList;
	string imageFileName;

	/* Read in the list of images for processing if the list argument in the command line is supplied */
	if(bFileList)
	{
		ifstream infile(inputFile);
		cout << "Batch file - " << string(inputFile) << endl << endl;
		appendToTestLog("Batch file - " + string(inputFile) + "\n");
		if(infile)
		{
			while (!infile.eof())
			{
				imageFileName = "";
				infile >> imageFileName;
				if(imageFileName != "\n" && imageFileName != "")
					imageFileList.push_back(imageFileName);
			}
		}
		infile.close();
	}
	/* Process the single file if the list argument in the command line is empty */
	else
	{
		imageFileList.push_back(static_cast<string>(inputFile));
		cout << "Image file - " << string(inputFile) << endl;
		appendToTestLog("Image file - " + string(inputFile) + "\n");
	}

	int NrOfPreDetectedSignatures = 0;
	int TotalDetectedSignatures = 0;
	char docCounter[5], pageCounter[5], zoneCounter[5];
	int zoneNumber = 0;

	list <DLSignatureCandidate> retrievedSigList;
	list <DLSignatureCandidate>::iterator sigCandidate_Iter;
	DLDocument::DLPagePtrIterator page_Iter;

	/* Detect signatures from input documents and store them in retrievedSigList */
	for(int imageIndex = 0; imageIndex < int(imageFileList.size()); imageIndex++)
	{
		string srcImageFullPathName = (string)imageFileList[imageIndex];
		DLImage docImage(srcImageFullPathName.c_str());
		DLDocument currentDoc;

		sprintf(docCounter, "%d", imageIndex+1);
		sprintf(pageCounter, "%d", 1);
		string docID = "Document" + static_cast<string>(docCounter);
		string pageID = "Page" + static_cast<string>(pageCounter);

		NrOfPreDetectedSignatures = 0;
		bool xmlFileFound = false;
		int fileExtPosition = int(srcImageFullPathName.find("."));
		int dirSepPosition = int(srcImageFullPathName.find_last_of(static_cast<string>(DL_DIR_SEPARATOR), fileExtPosition));
		string srcImageFileName, xmlFileName, xmlFilePath, xmlFullPathName;

		if(dirSepPosition != -1)	
		{
			srcImageFileName = srcImageFullPathName.substr(dirSepPosition);
			xmlFileName = srcImageFullPathName.substr(dirSepPosition, fileExtPosition);
			xmlFilePath = srcImageFullPathName.substr(0, dirSepPosition);
		}
		else 
		{
			srcImageFileName = srcImageFullPathName;
			xmlFileName = srcImageFullPathName.substr(0, fileExtPosition);
			xmlFilePath = "";
		}
		xmlFileName = xmlFileName + ".xml";
		xmlFullPathName = xmlFilePath + xmlFileName;

		ifstream xmlfile(xmlFileName.c_str());
		if(xmlfile)
		{
			/* Read in the xml file associated with the input document image if the xml file exists */
			xmlFileFound = true;
			cout << "Reading xml file - " << string(xmlFileName) << endl << endl;	
			DLxml xmlInput;
			currentDoc = xmlInput.dlReadFromXmlFile( xmlFileName, xmlFilePath );
			page_Iter = currentDoc.begin();

			if((*page_Iter)->dlHasZones())
			{
				/* iterate over all the zones and erase previous detection results */
				DLPage::DLZonePtrIterator zone_Iter1, zone_Iter2;
				for ( zone_Iter1 = (*page_Iter)->begin(); zone_Iter1 != (*page_Iter)->end(); )
				{
					DLTagList currentTagList = (*zone_Iter1)->dlGetAllTags();
					if(!currentTagList.dlIsTagListEmpty() && currentTagList.dlGetTag("gedi_type") == "DLDetectedSignature")
					{
						if(bErasePrevious) {
							zone_Iter2 = zone_Iter1;
							zone_Iter2++;
							(*page_Iter)->dlDeleteZone(*zone_Iter1);
							zone_Iter1 = zone_Iter2;
						}
						else {
							NrOfPreDetectedSignatures++;
							zone_Iter1++;
						}
					}
					else {
						zone_Iter1++;
					}
				}
			}
		}
		else
		{
			/* Create a single page document for storing detection information if there is no associated xml file */
			DLDocument thisDoc(docImage, pageID, docID);
			currentDoc = thisDoc;
			page_Iter = currentDoc.begin();
		}
		xmlfile.close();

		cout << "Processing " << srcImageFileName.c_str() << " ..." << endl;
		DLSignatureDetect sig(docImage, srcImageFullPathName.c_str(), DLSignatureDetect::PARAMETERSET(parameterSet), 1.5, false);
		list <DLSignatureCandidate> docSigList = sig.dlGetDetectedSignatures();

		if (!docSigList.empty())
		{
			// also create xml here
			for ( sigCandidate_Iter = docSigList.begin(), zoneNumber = NrOfPreDetectedSignatures+1; sigCandidate_Iter != docSigList.end(); sigCandidate_Iter++, zoneNumber++ )
			{
				sprintf(zoneCounter, "%d", zoneNumber);
				string zoneID = "DLDetectedSignature" + static_cast<string>(zoneCounter);

				DLZone *sigZone = new DLZone();
				*sigZone = sigCandidate_Iter->sigRegion;
				sigZone->dlSetZoneID(zoneID);
				sigZone->dlSetTag("id", "None");
				sigZone->dlSetTag("gedi_type", "DLDetectedSignature");
				(*page_Iter)->dlAppendZone(sigZone);

				// save the detected signature regions
				int sigOrigX = sigZone->dlGetZoneOrigin().x;
				int sigOrigY = sigZone->dlGetZoneOrigin().y;
				int sigWidth = sigZone->dlGetZoneWidth();
				int sigHeight = sigZone->dlGetZoneHeight();
				//string sigFileName = srcImageFileName.substr(0, srcImageFileName.find_last_of(".")) + "_signature.tif";
				//docImage.dlSubImage(sigOrigX, sigOrigY, sigWidth, sigHeight).dlSaveImage(sigFileName.c_str());

				retrievedSigList.push_back(*sigCandidate_Iter);
				TotalDetectedSignatures++;
			}
		}		

		/* Update existing xml file or create a new xml file containing the detection results */
		if(bOutputXML)
		{
			DLxml xmlOutput(currentDoc, srcImageFileName);	
			xmlOutput.dlSaveToXmlFile(xmlFullPathName);
			if(xmlFileFound) {
				cout << string(xmlFullPathName) << " updated" << endl << endl;
			}
			else {
				cout << string(xmlFullPathName) << " created" << endl << endl;
			}
		}
	}

	// sort the detected signature candidates from all documents in the list by their individual scores
	retrievedSigList.sort(compScores);

	cout << TotalDetectedSignatures << " signatures detected from " << imageIndex << " documents" << endl;
	
	appendToTestLog("\n\n\n******************************************************\n");
	appendToTestLog("**Test Summary\n");
	appendToTestLog("******************************************************\n\n");

	return 0;
}