#ifndef _DLSIGNATUREDETECT_H_
#define _DLSIGNATUREDETECT_H_

#include "DLImage.h"
#include "getLines.h"
#include "DLConnectedComponents.h"
#include "DLBitsPerPixelConverter.h"
#include "DLZone.h"
#include "DLConst.h"
#include "DLLogger.h"

#include <list>

using namespace std;

/// The support size of the image resize filter
#define RESIZE_FILTER_WIDTH 3.0
/// The threshold value for the edge intensity image
#define EDGE_MAG_THRE 50
/// The step value for a change in the edge orientation image
#define MIN_ORIENTATION_DIFF 8
/// The max space between parts of signatures during contour grouping
#define MAX_SIG_SPACING 37
/// The max height of signature at the initial coarse image scale
#define MAX_SIG_HEIGHT 600
/// The length of the largest connected strip in the same line (either horizontal or vertical)
#define MAX_DATARUN_LENGTH 50
/// This upper limit in the lambda value avoids the discrete effect when using estimated edge gradient information
#define LAMBDA_UPPER 10.0
/// This lower limit below which a value is treated as zero. This avoids the discrete effect when using estimated edge gradient information
#define SMALL_EPSILON 0.01
/// The lower bound of orthogality as encoded on the edge orientation image
#define ORTHOGAL_LOWER 58
/// The upper bound of orthogality as encoded on the edge orientation image
#define ORTHOGAL_UPPER 68
/// The number of histogram bins for analyzing orientation of a connected component
#define ORI_BINS 32
/// Lower bound of the height to width ratio of a connected component
#define CC_DIMENSION_RATIO_LOWER 0.15
/// Upper bound of the height to width ratio of a connected component
#define CC_DIMENSION_RATIO_UPPER 5.0
/// Lower bound of the height to width ratio of a grouped signature
#define SIG_DIMENSION_RATIO_LOWER 0.17
/// Upper bound of the height to width ratio of a grouped signature
#define SIG_DIMENSION_RATIO_UPPER 1.33
/// The max spatial density of a connected component
#define MAX_CC_DENSITY 0.5
/// The max number of connected components allowed for a signature
#define MAX_CC_PER_REGION 50

/**
*  \brief struct for storing a signature candidate and its associated information
*/
struct DLSignatureCandidate
{
	/// The spatial region correponding to the signature candidate
	DLZone sigRegion;
	/// Source image file name
	string sourceImageName;
	/// Computed score of the signature candidate region
	unsigned long scores;
	/// Computed score of its colinear neighbour
	unsigned long largestNeighScore;
	/// Bool type attribute used internally
    bool tested;
	/// String that stores any necessary type information for later processing needs
	string type;
};


/** 
 * \brief Comparison function for sorting text lines
 */
int comp (const void *entry1, const void *entry2);

/**
 * \class DLSignatureDetect
 *
 * \brief DLSignatureDetectDOCLIB implements a multi-scale signature detection and segmentation 
 * approach. The detection algorithm works in one of the two modes specified by the user.
 * It can either detect signatures by looking at the entire document image, or detect signatures
 * by exploring document context. When using document context, it effectively estimate statistics of
 * machine printed text lines and use them to locate the region below the main body of the document text, where
 * signatures typically appear. Using document context is more effective for machine printed documents.
 *
 * The idea of this signature detection approach is to capture the structural saliency of a signature
 * by measuring its dynamic curvature without recovering the tempo information. Once the most salient part
 * of the signature is identified, contour grouping is performed to obtain a complete and segmented signature.
 * As tested in large real world datasets, this approach is robust under large intra-class variations that
 * typically exhibited on unconstrained handwritting, and it is very effective across language differences.
 *
 */

class DLSignatureDetect
{
	public:

	/** 
	* \brief Parameter sets used for signature detection
	*/
	enum PARAMETERSET
	{
		DL_ENGLISH_PRINT = 1,
		DL_ARABIC_HANDWRITING = 2
	};

	/**
	* Default DLSignatureDetect Constructor. Initialize signature detector using the input image file
	* \param fname file name of the source image
	* \param parameterSet trained parameter sets used for signature detection. Refer to enumerator DLSIGNATUREDETECT_PARAMETERS for
	* the list of currently supported set
	* \param resizeRatio double the base factor for computing the initial coarest image scale
	* \param useContext bool type argument specifying whether to use document context for signature detection
	*/
	DLSignatureDetect(const char *fname, PARAMETERSET parameterSet=DL_ENGLISH_PRINT, double resizeRatio=2.0, bool useContext=true);

	/**
	* DLSignatureDetect Constructor. Initialize signature detector using an DLImage
	* \param srcImage DLImage of the source image
	* \param fname file name of the source image
	* \param parameterSet trained parameter sets used for signature detection. Refer to enumerator DLSIGNATUREDETECT_PARAMETERS for
	* the list of currently supported set
	* \param resizeRatio double the base factor for computing the initial coarest image scale
	* \param useContext bool type argument specifying whether to use document context for signature detection
	*/
	DLSignatureDetect(const DLImage &srcImage, const char *fname, PARAMETERSET parameterSet=DL_ENGLISH_PRINT, double resizeRatio=2.0, bool useContext=true);

	/**
	* DLSignatureDetect Destructor
	*/
	~DLSignatureDetect();

	/**
	* Return the list of detected signatures
	* \return list <DLSignatureCandidate> the list of detected signature candidates
	*/
	list <DLSignatureCandidate> dlGetDetectedSignatures() { return sigList; };

	/**
	* Check whether any signature is detected on the document page
	* \return true if at least one signature is detected
	*/
	bool dlGotSignatures() { return !sigList.empty(); };


  private:

	/**
	*  \brief struct for storing the language and dataset dependent parameters for signature detection
	*/
	struct DLSignatureDetectParameters
	{
		/// Detection threshold below which a candidate is rejected even if it has the highest score on the page
		unsigned long lowScoreThreshold;
		/// Detection threshold below which a candidate is rejected if a higher scored candidate is already found on the page
		unsigned long highScoreThreshold;
		/// Detection threshold for filtering a candidate which has scores uncharacteristic of a signature
		unsigned long ultraHighScoreThreshold;
		/// The min saliency score of a connected component
		unsigned long minCCScore;
		/// The max number of signatures per page. This is a dataset dependent property
		int maxSignaturePerPage;
		/// The max height of a connected component
		int maxCCHeight;
		/// The max width of a connected component
		int maxCCWidth;
		/// Min number of dark pixels in the connected component
		int minCCMass;
		/// Max spatial density of a grouped signature
		double maxSigDensity;
	};

	/**
	* Compute the proper initial image scale based on input image resolution and initialize the parameter sets
	* \param parameterSet trained parameter sets used for signature detection. Refer to enumerator DLSIGNATUREDETECT_PARAMETERS for
	* the list of currently supported set
	*/
	void dlInitializeSignatureDetect(DLSignatureDetect::PARAMETERSET parameterSet);
	  
	/**
	* Detect signatures from the entire document page or the contextually more important region on the page
	* based on user's option
	*/
	void dlDetectSignatures();
	  
	/**
	* Use the statistics of the machine printed text lines to locate the region below the main body of the document page,
	* where signatures typically appear. Using document context to locate the area on the page where signatures
	* appear is more effective on machine printed documents.
	*/
	void dlLocatePageFooter();
	  
	/**
	* Analyze the orientation information of a connected component for classification
	* \param magImageData DL_BYTE pointer to the data trunk storing the magnitude information of the computed edges
	* \param oriImageData DL_BYTE pointer to the data trunk storing the orientation information of the computed edges
	* \param cComponent the DLComponent to be analyzed
	*/
	void dlAnalyzeOriHistogram(DL_BYTE* magImageData, DL_BYTE* oriImageData, DLComponent &cComponent);

	/**
	* Compute the average inter-line distance between two vertically adjacent text lines
	* \return double the average inter-line distance between two vertically adjacent text lines
	*/
	double dlCalcAvgVerLineSeparation();

	/**
	* Remove a vertically connnected strip of edge pixels from the edge intensity image
	* \param magImageData DL_BYTE pointer to the data trunk storing the magnitude information of the computed edges
	* \param juncRow int the row to start from
	* \param juncCol int the column of the vertically connnected strip
	* \param mag_thres DL_BYTE the threshold used. The removal steps stop once an edge with intensity below this threshold is encountered
	*/
	void dlRemoveVerticalStrip(DL_BYTE* magImageData, int juncRow, int juncCol, DL_BYTE mag_thres);

	/**
	* Remove a horizontally connnected strip of edge pixels from the edge intensity image
	* \param magImageData DL_BYTE pointer to the data trunk storing the magnitude information of the computed edges
	* \param juncRow int the row of the horizontally connnected strip
	* \param juncCol int the column to start from
	* \param mag_thres DL_BYTE the threshold used. The removal steps stop once an edge with intensity below this threshold is encountered
	*/
	void dlRemoveHorizontalStrip(DL_BYTE* magImageData, int juncRow, int juncCol, DL_BYTE mag_thres);

	/**
	* Locate and remove those extremely large connected components from edge images
	* \param magImage DLImage of the edge intensity image
	* \param oriImage DLImage of the edge orientation image
	* \param edgeThresold DL_BYTE edge intensity threshold. Edges with intensity values less than this threshold
	* are treated as background pixels
	*/
	void dlFilterLargeCC(DLImage &magImage, DLImage &oriImage, DL_BYTE edgeThresold);

	/**
	* Remove the connected component from the edge intensity image
	* \param magImageData DL_BYTE pointer to the data trunk storing the magnitude information of the computed edges
	* \param cComponent the DLComponent to be removed
	*/
	void dlRemoveCC(DL_BYTE *magImageData, DLComponent &cComponent);

	/**
	* Compute the upper bound of parameter lambda for the two edge points at P1[x1, y1] and P2[x2, y2]
	* the gradient direction at points [x1, y1] and [x2, y2] are -p1/q1 and -p2/q2, respectively
	* \param x1 float the x coordinate of P1
	* \param y1 float the y coordinate of P1
	* \param p1 float internally used to compute the orientation at P1
	* \param q1 float internally used to compute the orientation at P1
	* \param x2 float the x coordinate of P2
	* \param y2 float the y coordinate of P2
	* \param p2 float internally used to compute the orientation at P2
	* \param q2 float internally used to compute the orientation at P2
	* \return float the computed upper bound of parameter lambda for the two edge points at [x1, y1] and [x2, y2]
	*/
	double dlFindLambdaRange(float x1,float y1,float p1,float q1,float x2,float y2,float p2,float q2);

	/**
	* Compute the spatial density of dark pixels in the input grayscale image region
	* \param grayImage DLImage of the input grayscale image
	* \param pixelValue DL_BYTE used a threshold for dark pixels. 
	* \return double the computed spatial density in the input grayscale image region
	*/
	double dlComputeDensity(DLImage &grayImage, DL_BYTE pixelValue);

	/// DLImage storing the source image
	DLImage sourceImage;

	/// String storing the file name of source image if available
	string	sourceImageName;

	/// The list of detected signature candidates stored in descending scores
	list <DLSignatureCandidate> sigList;

	/// The list of detected and segmented signatures
	list <DLSignatureCandidate> ccSigList;

	/// The language and dataset dependent parameter set. Each set of parameters have been optimized over representative datasets.
	DLSignatureDetectParameters sigDetectParameters;

	/// Bool indicating user's choice whether to use document context for detection
	bool useContext;

	/// Bool indicating whether image resolutions of source image are available
	bool resKnown;
    
	/// Horizontal resolution of source image
	double hRes;

	/// Vertical resolution of source image
	double vRes;

	/// Overall image resolution factoring in both horizontal and vertical resolutions of the source image
	double resolutionScale;

	/// Coarsest image scale computed from image resolutions of source image
	double resizeRatio;

	/// Width of the source document image
	int pageWidth;

	/// The height of the resized image at the coarest image scale
	int rows;

	/// The width of the resized image at the coarest image scale
	int cols;

	/// Pointer to the set of computed connected components
	DLConnectedComponents *cc;

	/// Data structure for storing statistics associated with text lines on the document page
	getLines *lineOjects;

	/// Data structure for storing statistics associated with text lines on the document page
	DLlineInfo *lineInfo;

	/// The average height of text lines
	double avgLineHeight;

	/// The average inter-line spacing between text lines
	double avgLineSeparation;

	/// The average width of text lines
	int widthAverage;

	/// The longest detected text line on the page
	int	widthLongestLine;

	/// The following parameters associate with the located contextually important region below the
	/// the main body of text where signatures typically appear
	/// The starting row of the located region
	int fromTop;

	/// The starting column of the located region. As a signature can be signed in a random fashion with regard to
	/// its horizontal position, the left margin is assumed to be 0.
	int leftMargin;

};//end class

#endif //_DLSIGDETECT_H_

