#include "DLStampDetectBatchTest.h"
#include "DLEllipseDetect.h"
#include "DLImage.h"
#include "DLDocument.h"
#include "DLPage.h"
#include "DLZone.h"
#include "DLTagList.h"
#include "DLTestManager.h"
#include "DLConst.h"
#include "DLxml.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

// The threshold of the edge strength, lower threshold is able to linking up weak edges
#define EDGE_MAG_THRE 50
#define RESIZE_RATIO 4.0

// The ratio between the lengths of the major axis and the minor axis for elliptic stamps
#define MAX_LENGTH_RATIO 3.0
#define MAX_PATH 260

DLStampDetectBatchTest::DLStampDetectBatchTest(char* name, int argc, char** argv) : DLTest(name, argc, argv)
{
	testName = name;
	dlArgc = argc;
	dlArgv = argv;
}

DLStampDetectBatchTest::~DLStampDetectBatchTest(void)
{
}

void DLStampDetectBatchTest::showHelp()
{
	/* Show command line help information */
	printf( "StampDetect [-x] [-e] [-min minArea] [-max maxArea] [-l] filename\n" );
	printf( "   -x   ---  Write detection results into an xml file for each input document.\n" );
	printf( "             It creates a new xml file if no associated xml file exists.\n" );
	printf( "   -e   ---  Remove all previous stamp detection results stored in xml files.\n" );
	printf( "             Write new results to xml file.\n" );
	printf( "   -min ---  Specify the minimum area of detected stamp in pixel squares.\n" );
	printf( "             Default value of 50000 is used if this argument is ignored.\n" );
	printf( "   -max ---  Specify the maximum area of detected stamp in pixel squares.\n" );
	printf( "             Default value of 200000 is used if this argument is ignored.\n" );
	printf( "   -l   ---  File containing the list of input images to execute\n" );
	printf( "   -h   ---  Show help at command line\n" );
}

int DLStampDetectBatchTest::go()
{
	char inputFile[MAX_PATH];
	char logFile[MAX_PATH];
	bool bFileList = false;
	bool bOutputXML = false;
	bool bErasePrevious = false;
	int minArea = 50000;
	int maxArea = 200000;

	/* Parse command line arguments */
	if(dlArgc == 1)
	{
		showHelp();
		return 0;
	}

	for(int i=1; i<dlArgc; i++)
	{
		if(dlArgv[i][0] != '-') break;
		++i;
		switch(tolower(dlArgv[i-1][1]))
		{
			case 'x':
				bOutputXML = true;
				i--;
				break;
			case 'e':
				bErasePrevious = true;
				i--;
				break;
			case 'min':
				minArea = atoi( dlArgv[i] );
				break;
			case 'max':
				maxArea = atoi( dlArgv[i] );
				break;
			case 'o':
				strcpy( logFile, dlArgv[i] );
				break;
			case 'l':
				bFileList = true;
				i--;
				break;
			case '?':
			case 'h':
				showHelp();
				return 0;
			default:
				printf( "Unable to interprete your command!\n" );
				showHelp();
				return -1;
		}
	}

	/* Show commmand line help panel if incorrect arguments are supplied */
	if( i != dlArgc-1 )
	{
		showHelp();
		return -1;
	}
	else if(minArea < 0 || maxArea < 0 || minArea >= maxArea )
	{
		printf( "Invalid area attributes specified!\n" );
		showHelp();
		return -1;
	}

	/* Grab the input file name specified at the command line */
	strcpy( inputFile, dlArgv[i] );

	vector <string> imageFileList;
	string imageFileName;

	/* Read in the list of images for processing if the list argument in the command line is supplied */
	if(bFileList)
	{
		ifstream infile(inputFile);
		cout << "Batch file - " << string(inputFile) << endl << endl;
		appendToTestLog("Batch file - " + string(inputFile) + "\n");
		if(infile)
		{
			while (!infile.eof())
			{
				imageFileName = "";
				infile >> imageFileName;
				if(imageFileName != "\n" && imageFileName != "")
					imageFileList.push_back(imageFileName);
			}
		}
		infile.close();
	}
	/* Process the single file if the list argument in the command line is empty */
	else
	{
		imageFileList.push_back(static_cast<string>(inputFile));
		cout << "Image file - " << string(inputFile) << endl;
		appendToTestLog("Image file - " + string(inputFile) + "\n");
	}

	int NrOfPreDetectedStamps = 0;
	int TotalDetectedStamps = 0;
	char docCounter[5], pageCounter[5], zoneCounter[5];
	int zoneNumber = 0;

	list <DLEllipseCandidate> detectedEllipticStampList;
	list <DLEllipseCandidate>::iterator ellip_Iter;

	DLDocument::DLPagePtrIterator page_Iter;
	string srcImageFullPathName;

	/* Detect stamps from input document images and store them */
	for(int imageIndex = 0; imageIndex < int(imageFileList.size()); imageIndex++)
	{
		srcImageFullPathName = (string)imageFileList[imageIndex];
		DLImage docImage(srcImageFullPathName.c_str());
		DLDocument currentDoc;

		sprintf(docCounter, "%d", imageIndex+1);
		sprintf(pageCounter, "%d", 1);
		string docID = "Document" + static_cast<string>(docCounter);
		string pageID = "Page" + static_cast<string>(pageCounter);

		NrOfPreDetectedStamps = 0;
		bool xmlFileFound = false;
		int fileExtPosition = int(srcImageFullPathName.find("."));
		int dirSepPosition = srcImageFullPathName.find_last_of(static_cast<string>(DL_DIR_SEPARATOR), fileExtPosition);
		string srcImageFileName, xmlFileName, xmlFilePath, xmlFullPathName;

		if(dirSepPosition != -1)	
		{
			srcImageFileName = srcImageFullPathName.substr(dirSepPosition);
			xmlFileName = srcImageFullPathName.substr(dirSepPosition, fileExtPosition);
			xmlFilePath = srcImageFullPathName.substr(0, dirSepPosition);
		}
		else 
		{
			srcImageFileName = srcImageFullPathName;
			xmlFileName = srcImageFullPathName.substr(0, fileExtPosition);
			xmlFilePath = "";
		}
		xmlFileName = xmlFileName + ".xml";
		xmlFullPathName = xmlFilePath + xmlFileName;

		ifstream xmlfile(xmlFileName.c_str());
		if(xmlfile)
		{
			/* Read in the xml file associated with the input document image if the xml file exists */
			xmlFileFound = true;
			cout << "Reading xml file - " << string(xmlFileName) << endl << endl;	
			DLxml xmlInput;
			currentDoc = xmlInput.dlReadFromXmlFile( xmlFileName, xmlFilePath );
			page_Iter = currentDoc.begin();

			if((*page_Iter)->dlHasZones())
			{
				/* iterate over all the zones and erase previous detection results */
				DLPage::DLZonePtrIterator zone_Iter1, zone_Iter2;
				for ( zone_Iter1 = (*page_Iter)->begin(); zone_Iter1 != (*page_Iter)->end(); )
				{
					DLTagList currentTagList = (*zone_Iter1)->dlGetAllTags();
					if(!currentTagList.dlIsTagListEmpty() && currentTagList.dlGetTag("gedi_type") == "DLStamp")
					{
						if(bErasePrevious) {
							zone_Iter2 = zone_Iter1;
							zone_Iter2++;
							(*page_Iter)->dlDeleteZone(*zone_Iter1);
							zone_Iter1 = zone_Iter2;
						}
						else {
							NrOfPreDetectedStamps++;
							zone_Iter1++;
						}
					}
					else {
						zone_Iter1++;
					}
				}
			}
		}
		else
		{
			/* Create a single page document for storing detection information if there is no associated xml file */
			DLDocument thisDoc(docImage, pageID, docID);
			currentDoc = thisDoc;
			page_Iter = currentDoc.begin();
		}
		xmlfile.close();

		cout << "Processing " << srcImageFileName.c_str() << " ..." << endl;

		/* Detect elliptic stamps with the specified bounds in area and ratio between major and minor axes */
		DLEllipseDetect ellipseDetector( docImage, int(EDGE_MAG_THRE), RESIZE_RATIO, minArea, maxArea, MAX_LENGTH_RATIO, 0, 1.0, 0, 1.0, 100, 100, 100);
		list <DLEllipseCandidate> ellipticObjList = ellipseDetector.dlGetDetectedEllipses();

		if (!ellipticObjList.empty())
		{
			for ( ellip_Iter = ellipticObjList.begin(), zoneNumber = NrOfPreDetectedStamps+1; ellip_Iter != ellipticObjList.end(); ellip_Iter++, zoneNumber++ )
			{
				sprintf(zoneCounter, "%d", zoneNumber);
				string zoneID = "DLDetectedStamp" + static_cast<string>(zoneCounter);

				DLZone *stampZone = new DLZone();;
				*stampZone = ellip_Iter->ellipseRegion;
				stampZone->dlSetZoneID(zoneID);
				stampZone->dlSetTag("id", "None");
				stampZone->dlSetTag("gedi_type", "DLDetectedStamp");
				stampZone->dlSetTag("StampType", "EllipticStamp");
				(*page_Iter)->dlAppendZone(stampZone);
				detectedEllipticStampList.push_back(*ellip_Iter);
				TotalDetectedStamps++;
			}
		}
		
		/* Update existing xml file or create a new xml file containing the detection results */
		if(bOutputXML)
		{
			DLxml xmlOutput(currentDoc, srcImageFileName);	
			xmlOutput.dlSaveToXmlFile(xmlFullPathName);
			if(xmlFileFound) {
				cout << string(xmlFullPathName) << " updated" << endl << endl;
			}
			else {
				cout << string(xmlFullPathName) << " created" << endl << endl;
			}
		}
	}

	cout << TotalDetectedStamps << " stamps detected from " << imageIndex << " documents" << endl;
	
	appendToTestLog("\n\n\n******************************************************\n");
	appendToTestLog("**Test Summary\n");
	appendToTestLog("******************************************************\n\n");

	return 0;

}



