#ifndef _DLBMPIMAGE_H_
#define _DLBMPIMAGE_H_

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLIOStream.h"
#include "DLSingletonDestroyer.h"
/**
 * \brief BMP image I/O class.
 *
 * Implements loading and saving for the BMP image format, and is used by
 * DLImageFactory. See DLImage for the typical interface for loading and saving
 * images.
 */
class DLBMPImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:
   /**
    * Default DLBMPImage Constructor
    */
   DLBMPImage();

   ///Instance of DLBMPImage
   static DLBMPImage* bmpImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLBMPImage> _destroyer;

   protected:
   /**
    * Default DLBMPImage Destructor
    */
   virtual ~DLBMPImage();

   /**
    * dlStream is a private IO object that handles the reads
    * and writes.  BMP uses this object to read/write the 
    * image output source
    * \param dlStream Doclib IO stream for read/write
    * \return dlimage
    */
   DLImage dlLoadImage(DLIOStream &dlStream);

   /**
    * save a dlimage to the dlStream
    * \param dlStream Doclib IO stream for read/write
    * \param dlimage DLImage object to be saved
    */
   void dlSaveImage(DLIOStream &dlStream, const DLImage& dlimage);

   public:
   /// \return A handle to the  DLBMPImage object
   static DLBMPImage * createImageReader();

   /**
    * Load BMP Image
    * \param[in] filename name of file to be loaded
    * \param[out] dlimage DLImage object passed by reference;
    * will be overwritten to contain image from BMP file.
    */
   void dlLoadImage(const char* filename, DLImage& dlimage);

   /**
    * Load BMP Image
    * \param filename name of BMP image file to be opened
    * \return DLImage of BMP image file
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * Load BMP Image from memory
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize size of memory buffer in bytes
    * \return DLImage for the BMP Image
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Loads a particular page of a multiple page
    * image into a DLImage
    * \param filename File name of image to be opened
    * \param pageNumber page number to be opened
    * \return page as a DLImage
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * Loads a particular page of a multiple page
    * image into a DLImage
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage to be returned
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * Save BMP Image
    * \param filename name of file to be saved
    * \param dlimage BMP Image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& dlimage);

   /**
    * Save BMP Image to a memory buffer
    * \param[out] nSize size of the buffer needed by the image
    * \param[in] dlimage BMP Image to be saved
    * \return pointer to the beginning of the saved image buffer
    */
   void* dlSaveImageToMem(int& nSize, const DLImage &dlimage);

   /**
    * Determines if the image file is in the BMP format
    * and is supported for loading by DLBMPImage.
    * \param filename name of file to check
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if the memory buffer is in the BMP format
    * and is supported for loading by DLBMPImage.
    * \param pMemLoc pointer to beginning of buffer to check
    * \param size size of buffer in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);
};//end class

static DLBMPImage * dlBmpProcessor = DLBMPImage::createImageReader();

#endif //_DLBMPIMAGE_H_


