#ifndef DLBASEIMAGE_H
#define DLBASEIMAGE_H

#include "DLImage.h"
#include <vector>

using namespace std;

/**
 * \brief An abstract base class for all image types.
 *
 * DLBaseImage is used internally by DOCLIB to support the loading and saving
 * of images in a variety of file formats. First see DLImage for the typical DOCLIB
 * programming interface for loading and saving images.
 *
 * As an abstract base class, no objects of class DLBaseImage are meant to 
 * be instantiated.
 *
 * Any DOCLIB supported image format (i.e. TIFF, JPEG, PPM, etc) needs
 * to be derived from this class and implement its interface to ensure the support of common
 * functionality and enable the plug-in image object design.
 *
 * DLBaseImage is used by DLImageFactory: see there for details about the design
 * of the image loading and saving system. See also the various derived image format
 * classes for specialized load and save functions specific to that image format.
 */
class DLBaseImage
{

   public:

   /**
    * Returns the name of the image object
    * \return image name
    */
   string getImageName();

   /**
    * Returns the file name extensions supported by the 
    * image format object as a comma-separated string
    * (e.g. <tt>"tiff,tif"</tt>)
    * \return string of image extensions
    */
   string getExtension();

   /**
    * dlSupportImageExt determines if the image format objeect supports
    * the given image extension.
    * \return boolean: true if supported
    */
   bool dlSupportImageExt(string ext);

   /**
    * This is an abstract function to determine if the given 
    * file is supported by the image format object.  All derived
    * classes must implement this function because only that particular
    * image object will know its own image format. Derived classes
    * will check for things such as magic numbers or typical header
    * information to make their determination.
    * \param filename file to check
    * \return boolean: true if supported
    */
   virtual bool dlSupportImageFile(const char* filename) = 0;

   /**
    * This is an abstract function to determine if the given 
    * buffer is x supported by the image format object.  All derived
    * classes must implement this function because only that particular
    * image object will know its own image format. Derived classes
    * will check for things such as magic numbers or typical header
    * information to make their determination.
    * \param pMemLoc pointer to start of memory buffer
    * \param size size of memory buffer in bytes
    * \return boolean: true if supported
    */
   virtual bool dlSupportImageBuffer(const void* pMemLoc, int size) = 0;

   /**
    * This is an abstract save function to save a dlImage to a file. 
    * This function must be implemented by all child classes to save 
    * the image into their own image format.
    *
    * \param filename file name to save the image to
    * \param image
    */
   virtual void dlSaveImage(const char* filename, const DLImage& image)= 0;

   /**
    * This is an abstract save function to save a dlImage to a memory buffer. 
    * This function must be implemented by all child classes to save 
    * the image into their own image format.
    *
    * \param[out] nSize int size of buffer is returned
    * \param[in] dlimage
    * \return pointer to memory buffer in which the image was saved
    */
   virtual void* dlSaveImageToMem(int& nSize, const DLImage& dlimage)= 0;

   /**
    * This is an abstract load function to load a dlImage from a file. 
    * This function must be implemented by all child classes to load 
    * the image from their own image format into a DLImage.
    * \param filename file name to open
    * \return DLImage 
    */	
   virtual DLImage dlLoadImage(const char* filename) = 0;

   /**
    * This is an abstract load function to load a dlImage from a memory block. 
    * This function must be implemented by all child classes to load 
    * the image from their own image format into a DLImage.
    * \param pMemLoc pointer to start of memory block to be opened
    * \param nSize size of memory block in bytes
    * \return DLImage 
    */	
   virtual DLImage dlLoadImage(const void* pMemLoc, int nSize) = 0;

   /**
    * Tokenize takes a string and split the string into multiple
    * sub strings separated by a given token.
    * \param str string to be tokenize
    * \param tokens stores the result of tokens in a vector
    * \param delimiters delimiter used to divide the string
    */
   static void Tokenize(const string& str,
                        vector<string>& tokens, const string& delimiters = " ");

   /**
    * This is an abstract function to determine the total number 
    * of pages in a give image buffer.  All child classes must implement this
    * function.  If image does not support multiple pages, have 
    * function return "1".  
    *
    * \param pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   virtual int dlGetTotalPages(void* pMemLoc, int nSize)=0;

   /**
    * This is an abstract function to determine the total number 
    * of pages in an image file.  All child classes must implement this
    * function.  If image does not support multiple pages, have 
    * function return "1".  
    * \param  filename file name
    * \return number of pages
    */
   virtual int dlGetTotalPages(const char* filename)= 0;

   /**
    * dlLoadImageAtPage is an abstract function to load a particular 
    * page of an image from a file.
    * \param filename Image file name to be opened
    * \param pageNumber page number to be opened
    * \return DLImage of desired page
    */
   virtual DLImage dlLoadImageAtPage(const char* filename, int pageNumber) = 0;

   /**
    * dlLoadImageAtPage is an abstract function to load a particular 
    * page of an image from a memory buffer.
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber page number to be opened
    * \return DLImage of desired page
    */
   virtual DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber) = 0;

   /** 
    * dlSetRegistrationPriority sets the registration priority of the 
    * image object.  This is a common function among all 
    * supported types.
    */ 
   void dlSetRegistrationPriority(int priority){regPriority = priority;}

   /**
    * dlGetRegPriority returns the registration priority of the 
    * image object.  This is a common function among all 
    * supported types.
    */
   int dlGetRegistrationPriority() const {return regPriority;}
   /**
    * Destructor
    **/
   virtual ~DLBaseImage(void);
   protected:
   ///**
   //* dlSetImageData function is used to set the private image data 
   //* in DLImage.  All child objects must call this function to 
   //* set the image data.
   //*
   //* \param image DLimage object to set the data to
   //* \param imageData byte data to set in DLImage
   //**/
   //void dlSetImageData(DLImage& image, DL_BYTE * imageData);

   /**
    * Constructor 
    * \param imageName A tag to identify the image format object
    * \param exts Comma-separated string of file extensions
    * supported by the class (e.g. <tt>"jpg,jpeg,jfif"</tt>)
    * \param regPriorityLevel the default registration priority
    * level is hard coded to 5. This value could be overwritten by any int value. 
    **/
   DLBaseImage(string imageName, string exts, int regPriorityLevel = 5 );

   private:
   ///Name of the image object (i.e. Jpeg)
   string name;   
   /// supported extentions
   string extensions;
   /**
    * Priority of the DLImageFactory Registration.  
    * This variable is used in DLImageFactory to determin
    * the ordering of the supported image objects.
    */
   int regPriority;

};


#endif //DLBASEIMAGE_H
