#ifndef _DLEXCEPTION_H_
#define _DLEXCEPTION_H_

#include "DLException.h"
#include <string>
#include <stdexcept>
using namespace std;

/**
 * Enum of exception types.
 * This ID is used when throwing an 
 * DLException.  User can  use this ID to 
 * identify the type of exception being 
 * thrown.
 */
enum DLExceptionCodes {
   DL_IO_EXCEPTION = 1,
   DL_FILE_NOT_FOUND_EXCEPTION = 2,
   /// some functionality is not yet implemented; for example, a function stub which is
   /// incomplete
   DL_NOT_IMPLEMENTED_EXCEPTION = 3,
   DL_UNKNOWN_FILE_FORMAT_EXCEPTION = 4,
   /// a pointer to be dereferenced turns out to be NULL
   DL_NULL_POINTER_EXCEPTION = 5,
   DL_UNKNOWN_OBJECT_EXCEPTION = 6,
   DL_WRONG_FORMAT_EXCEPTION = 7,
   DL_NOT_SUPPORTED_IMAGE_EXCEPTION = 8,
   DL_OUT_IMAGE_BOUNDARY_EXCEPTION = 9,
   DL_OUT_CHANNEL_NUMBER_EXCEPTION = 10,
   DL_NOT_SUPPORTED_EXCEPTION = 11,
   DL_UNKNOWN_TAG_EXCEPTION = 12,
   DL_MEMORY_EXCEPTION = 13,
   DL_PARAMETERS_EXCEPTION = 14,
   DL_INVALID_PAGE_NUMBER= 15,
   /// a DOCLIB function has been deprecated and will be removed in a future release
   DL_DEPRECATED_EXCEPTION = 16,
   DL_INDEX_OUT_OF_BOUND = 17
};

/**
 * \brief Error message object used to deliver error messages in DOCLIB
 * 
 * DLException is a DOCLIB user-defined exception message object that will be thrown
 * when an error has been detected in the DOCLIB library. The DLException contains 
 * the error message and an ID (from the enum DLExceptionCodes) that describes the reason of the 
 * exception. 
 *
 * DOCLIB programmers who are creating libraries for others' use should report
 * errors by throwing a DLException, or a class derived from DLException.
 *
 * Programmers using the DOCLIB API should expect that an exception may be thrown
 * when invoking DOCLIB functionality; for example, opening an image file into a
 * DLImage should be placed inside a \c try block, since if, for instance, 
 * the desired image file does not exist, the DLImage::dlLoadImage() code will 
 * throw a DLException.
 *
 * Note: Need to free any dynamically-allocated memory not handled by a destructor
 * before throwing an exception.  
 *
 * 
 * Typical usage:
 * \code 
 * #include <exception>
 * #include "DLException.h"
 *  
 * try {
 *   callSomeFunction();
 * }
 * 
 *  // Catch DLException referance
 * catch(const DLException &ex)
 * {
 *   // Get the error message
 *   // and print to screen
 *   cout << ex.what();  
 *     
 *   // Get the exception ID.
 *   DLExceptionCodes errorID = ex.dlGetErrorID();  
 * } 
 * // Catch any exception derived from the standard C++ exception
 * catch(const std::exception &ex)
 * {
 *   // Get the error message
 *   // and print to screen
 *   cout << ex.what();  
 * } 
 * // Catch all unknown excpetions
 * catch (...)
 * {
 *   cout << "unknown exception!!"; 
 * }
 *  :
 *  :
 *  :   
 * void callSomeFunction()
 * {  
 *   // Test some condition.  If condition is not right,
 *   // Throw a DLException
 *   if( something wrong )
 *   // Clean up memory  
 *   {   // __FILE__ is a macro that will insert the current file name
 *       // __LINE__ is a macro that will insert the current line number
 *     throw DLException(DL_IO_EXCEPTION, "Cannot find input file", __FILE__, __LINE__);  
 *   }
 * }
 *
 * \endcode
 * 
 */
class DLException : public std::exception
{
  
   public:
   /**
    * Default DLException Constructor
    */
   DLException();
  
   /**
    *  Copy  Constructor for copying the DLException
    */
   DLException(const DLException & orig);
  
   /**
    *  Assignment operator
    * \param right DLException to be copied
    */
   const DLException & operator=(const DLException & right);
  
   /**
    * DLException Constructor that takes in 
    * the error code, error message, filename and line number.
    * \param errCode exception code; can be one of the following:
    * <ul><li>DL_IO_EXCEPTION 
    * <li>DL_FILE_NOT_FOUND_EXCEPTION 
    * <li>DL_NOT_IMPLEMENTED_EXCEPTION 
    * <li>DL_UNKNOWN_FILE_FORMAT_EXCEPTION 
    * <li>DL_NULL_POINTER_EXCEPTION 
    * <li>DL_UNKNOWN_OBJECT_EXCEPTION 
    * <li>DL_WRONG_FORMAT_EXCEPTION 
    * <li>DL_NOT_SUPPORTED_IMAGE_EXCEPTION 
    * <li>DL_OUT_IMAGE_BOUNDARY_EXCEPTION 
    * <li>DL_OUT_CHANNEL_NUMBER_EXCEPTION
    * <li>DL_NOT_SUPPORTED_EXCEPTION
    * <li>DL_UNKNOWN_TAG_EXCEPTION 
    * <li>DL_MEMORY_EXCEPTION
    * <li>DL_PARAMETERS_EXCEPTION
    * <li>DL_INVALID_PAGE_NUMBER
    * <li>DL_DEPRECATED_EXCEPTION
    * <li>DL_INDEX_OUT_OF_BOUND
    * </ul>
    * 
    * \param str error message string
    * \param filename Source code file name where the exception is being thrown.
    *  This is an optional field.
    *  Should use __FILE__ (a macro for the current file name).
    * \param lineNum Line number where the exception is being thrown.  This is an optional field.
    * Should use __LINE__ (a macro for the line number of the throw statement)
    */
   DLException(DLExceptionCodes errCode, const char* str, const char* filename = 0, const int lineNum = 0);
  
   /**
    * \return error message string
    */
   string dlGetErrorMsg() const;
  
   /**
    * Gets the error ID  
    * \return error ID code; one of the following:
    * <ul><li>DL_IO_EXCEPTION 
    * <li>DL_FILE_NOT_FOUND_EXCEPTION 
    * <li>DL_NOT_IMPLEMENTED_EXCEPTION 
    * <li>DL_UNKNOWN_FILE_FORMAT_EXCEPTION 
    * <li>DL_NULL_POINTER_EXCEPTION 
    * <li>DL_UNKNOWN_OBJECT_EXCEPTION 
    * <li>DL_WRONG_FORMAT_EXCEPTION 
    * <li>DL_NOT_SUPPORTED_IMAGE_EXCEPTION 
    * <li>DL_OUT_IMAGE_BOUNDARY_EXCEPTION 
    * <li>DL_OUT_CHANNEL_NUMBER_EXCEPTION
    * <li>DL_NOT_SUPPORTED_EXCEPTION
    * <li>DL_UNKNOWN_TAG_EXCEPTION 
    * <li>DL_MEMORY_EXCEPTION
    * <li>DL_PARAMETERS_EXCEPTION
    * <li>DL_INVALID_PAGE_NUMBER
    * <li>DL_DEPRECATED_EXCEPTION
    * <li>DL_INDEX_OUT_OF_BOUND
    * </ul>
    */
   DLExceptionCodes dlGetErrorID() const;

   /**
    * An inherited function from std::exception.  
    * \return error message as C-style string
    */
   const char *what() const throw();
  
   /**
    * Default Destructor
    */
   virtual ~DLException() throw ();
  
  
   protected:
   ///Error message
   string errorMessage;
   ///Exception ID
   DLExceptionCodes errorID;
	
};//end class
#endif //_DLEXCEPTION_H_
