
#ifndef DLIOSTREAM_H
#define DLIOSTREAM_H
#include "DLUtils.h"
#include <iostream>
using namespace std;

/**
 * \brief DLIOStream is a Base class for the DOCLIB 
 * I/O package (i.e. Read/Write file, Read/Write memory and etc.)
 *
 * DLIOStream implements the common features and 
 * defines all the required interfaces (minimum features)
 * that all child objects must implement to allow the 
 * ease of swapping in/out other DOCLIB I/O objects.
 *
 * Currently, Some of the DOCLIB supported image objects (PPM, BMP, PBM)
 * are using DLIOStream to read and write to and from file/memory.   
 *
 * DLIOStream also take care of the byte order (endian) 
 * of the current system when reading for int/short
 * data types.
 */

class DLIOStream
{
   public:

   /*
    * Constructor.  Sets the endian byte order for the current system
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * 
    */
   DLIOStream(){this->sys_endian = DLUtils::testByteOrder();
   };


   /**
    * dlRead is a pure virtual function to read up to a number of (size) 
    * bytes form the DOCLIB I/O stream into the buffer. dlRead will read up
    * to the max size or end of file.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read.
    * \return size of bytes read
    */	
   virtual int dlRead(void* buff, int size) = 0;
   /**
    * dlGets is a pure virtual function to read up to a number of (size) 
    * bytes form the DOCLIB I/O stream into the buffer. dlGets will read up
    * to the max size or until line feed or end of file.
    * The buffer must be allocated prior calling the dlRead() function.
    * The number of bytes read in is returned. 
    *
    * \param buff buffer to store the data
    * \param size max buffer size to read
    * \return size of bytes read
    */	
   virtual int dlGets(void* buff, int size) = 0;

   /**
    * dlGetLine is a pure virtual function to read a line form the 
    * DOCLIB I/O stream. dlGetLine will read up to an end of line or 
    * DL_PPM_MAX_LINE_LENGTH (256) or end of file. A string of
    * the read in line will be returned.
    *
    * \return string that has been read in
    */
   virtual string dlGetLine () = 0;

   /**
    * dlGetInt is a pure virtual function to read an int "sizeof(int) bytes"
    * form the DOCLIB I/O stream. dlGetInt will convert the byte order 
    * of the current int value into the indicated byte order.  
    * The read in "int" value is returned. 
    *
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>

    * \return read in int value
    */	
   virtual int dlGetInt(DLEndian endian) = 0;

   /**
    * dlGetShort is a pure virtual function to read a short "sizeof(short) bytes"
    * form the DOCLIB I/O stream. dlGetShort will convert the byte order 
    * of the current short value into the indicated byte order.  
    * The read in "short" value is returned. 
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>

    * \return read in short value 
    */	
   virtual short dlGetShort(DLEndian endian)  = 0;
   /**
    * dlWriteInt is a pure virtual function to write an int to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * int value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param intVal int value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    */	
   virtual void dlWriteInt(int intVal, DLEndian endian) = 0;
   /**
    * dlWriteInt is a pure virtual function to write a short to 
    * the DOCLIB I/O stream.  dlWriteInt will convert the 
    * short value into the indicated byte order and then writes 
    * it out to the DOCLIB I/O stream.
    *
    * \param shortVal short value to write
    * \param endian desired byte order
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>

   */	
   virtual void dlWriteShort(short shortVal, DLEndian endian) = 0;

   /**
    * dlWrite is a pure virtual function to write a buffer with
    * size to the DOCLIB I/O stream.
    * \param buff buffer to write
    * \param size buffer size to write
    * \return size 
    */	
   virtual int dlWrite(void* buff, int size) = 0;  

   /**
    * dlClose is a pure virtual function to close the current
    * DOCLIB I/O stream. 
    *
    * \return returns 1 if successfully closed the DOCLIB I/O stream.  Otherwise, returns 0 
    */	
   virtual int dlClose () = 0;
   /**
    * dlFlush is a pure virtual function to flush the current
    * DOCLIB I/O stream. 
    *    
    * \return returns 1 if successfully flushes the DOCLIB I/O stream.  Otherwise, returns 0 
    */	
   virtual int dlFlush() = 0;


   protected:
   ///Destructor
   virtual ~DLIOStream(){};
   /**
    * Stors the current system's byte order (endian).
    *  <ul>
    *  <li> DL_LE - Little Endian
    *  <li> DL_BE - Big Endian
    *  </ul>
    * This value is set during construction of the IOStream.
    */
   DLEndian sys_endian;
};
#endif //DLIOSTREAM_H
