#ifndef _DLIMAGE_H_
#define _DLIMAGE_H_

#include "DLImageHead.h"
#include "DLException.h"
#include "DLImageStats.h"
#include "DLLogger.h"

#include "DLConst.h"
#include "DLTypes.h"
#include "DLPoint.h"
#include "DLColor.h"

#include <iostream>
#include <string>
#include <vector>

using namespace std;

class DLBaseImage;
class DLZone;
/**
 * \brief The DLImage object is the entry point for performing image processing capabilities
 * supported by DOCLIB. 
 *
 * It serves as the core for all future DOCLIB capabilites that
 * will be hooked in. DLDocument and DLVideo classes will inherit the features of DLImage
 * as they are developed.
 *
 * An image in memory or on disk is read into DOCLIB's internal data format. All operations are 
 * performed on this format, completely independent of image type.
 * The DLImage object is composed of the image header (see DLImageHead, which stores 
 * basic metadata such as height, width and resolution, as well as DLTagList, 
 * for storing user-defined metadata) and the image data itself (see dlGetSafeImageData()
 * and dlGetImageData() for data access). The very easiest way to load an image and get
 * started is: 
 * \code
 * DLImage i("test.jpg");
 * \endcode
 *
 * Several basic image processing functions are included in the interface to DLImage. 
 * Certain other functions are implemented in other classes as static functions; see also
 * DLBitsPerPixelConverter, DLSkeleton and DLSkew.
 */
class DLImage 
{

   /**
    * Declare DLBitsPerPixelConverter as friend to allow all 
    * DLBitsPerPixelConverter functions to access private DLImage members
    */
   friend class DLBitsPerPixelConverter;

   public:

   /**
    * Default DLImage Constructor. Initializes image data to \c NULL.
    */
   DLImage();

   /**
    * DLImage Copy Constructor.
    * \param orig DLImage to copy
    */
   DLImage( const DLImage & orig);

   /**
    * DLImage Constructor: takes a image data pointer and an image info header.
    * \param data DL_BYTE pointer to image data.  Caller will allocate this memory
    * and pass the memory block to the constructor.  DLImage will be responsible 
    * for freeing this memory.
    * \param imageHead image header information
    * \throw DL_NULL_POINTER_EXCEPTION when data is \c NULL
    */
   DLImage( DL_BYTE *data, const DLImageHead &imageHead);  

   /**
    * DLImage Constructor: creates an empty image (white background) with width, height, and depth header information.
    * Memory for the image data will be allocated.
    * \param w width of image
    * \param h height of image
    * \param depth Number of bits per image pixel; can be one of:
    * \arg DLImageHead::DL_BITDEPTH_1
    * \arg DLImageHead::DL_BITDEPTH_8
    * \arg DLImageHead::DL_BITDEPTH_24
    */
   DLImage(int w, int h, DLImageHead::DLBitDepths depth); 

   /**
    * DLImage Constructor: creates an empty image (white background) with width, height, and depth header information.
    * Memory for the image data will be allocated.
    * \param w width of image
    * \param h height of image
    * \param depth Number of bits per image pixel; can be one of:
    * \arg ::DL_BITDEPTH_1
    * \arg ::DL_BITDEPTH_8
    * \arg ::DL_BITDEPTH_24
	* \deprecated Use DLImage (int w, int h, DLImageHead::DLBitDepths depth) instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   DLImage(int w, int h, int depth); 


   /**
    * DLImage Constructor: Loads an image from a file.
    * If the image file format is known, a pointer to one of the image format
    * objects (see DLBaseImage) can  be passed in;
    * however, if the second parameter is not used,
    * DOCLIB will try and open using all format types. 
    * If an image is not
    * recognized by any of DOCLIB's supported types, an exception will be thrown.
    * Note that each image format object
    * provides a function createImageReader() which returns a handle of 
    * and image format object. Also, for convenience, globally-accessible handle
    * variables are provided for each format. These are:
    * <UL>
    * <LI> ::dlBmpProcessor
    * <LI> dlGifProcessor
    * <LI> dlJpgProcessor
    * <LI> dlPbmProcessor
    * <LI> dlPngProcessor
    * <LI> dlJpmProcessor
    * <LI> dlTiffProcessor
    * </ul>
    * Here are examples of each loading method:
    * \code
    * DLImage i1("test.jpg"); // load an image; DOCLIB determines file format
    * DLImage i2("image2", dlPngProcessor); // force DOCLIB to load as PNG
    * DLImage i3("image2", DLPNGImage::createImageReader()); // force DOCLIB to load as PNG
    * \endcode
    * \param filename name of file to open
    * \param imageType (optional) handle of an image format object
    */
   DLImage(const char *filename, DLBaseImage* imageType = NULL);

   /**
    * DLImage Constructor: Load an image from a memory block.
    * The memory block must be the memory image of an image file in one
    * of the file formats supported by DOCLIB.
    * If the image file format is known, a pointer to one of the image format
    * objects (see DLBaseImage) can  be passed in;
    * however, if the second parameter is not used,
    * DOCLIB will try and open using all format types. 
    * If an image is not
    * recognized by any of DOCLIB's supported types, an exception will be thrown.
    * \param pMemLoc pointer to beginning of memory block
    * \param nSize size of the memory block in bytes
    * \param imageType (optional) pointer to image format object
    */
   DLImage(const void *pMemLoc,int nSize, DLBaseImage* imageType = NULL);
   /**
    * Default DLImage Destructor. Deletes image data and reference.
    */
   virtual ~DLImage();

   /**
    * A selection of possible directions.
    */
	enum DLFlipDirection {
		DL_HORIZONTAL = 1,
		DL_VERTICAL = 2
	};

   /**
	* A selection of possible color channels
	*/
	enum DLColorChannels {
		DL_INVALID_CHANNEL = -1,
		DL_RED_CHANNEL = 0,
		DL_GREEN_CHANNEL = 1,
		DL_BLUE_CHANNEL = 2
	};
   /**
    * Assignment operator: make a copy of an image.
    * \code
    * DLImage i("test.jpg");
    * DLImage j = i; // copy image \c i to image \c j
    * \endcode
    * \throw DL_MEMORY_EXCEPTION when a self assignment is attempted
    */
   DLImage& operator = (const DLImage &image);
	
   /**
    * Opens an image written to disk.
    * If the image file format is known, a pointer to one of the image format
    * objects (see DLBaseImage) can  be passed in;
    * however, if the second parameter is not used,
    * DOCLIB will try and open using all format types. 
    * If an image is not
    * recognized by any of DOCLIB's supported types, an exception will be thrown.
    * \param filename name of file to open
    * \param imageType (optional) pointer to image format object
    */
   void dlLoadImage(const char* filename, DLBaseImage* imageType = NULL);


   /**
    * Opens an image from memory.
    * The memory block must be the memory image of an image file in one
    * of the file formats supported by DOCLIB.
    * If the image file format is known, a pointer to one of the image format
    * objects (see DLBaseImage) can  be passed in;
    * however, if the second parameter is not used,
    * DOCLIB will try and open using all format types. 
    * If an image is not
    * recognized by any of DOCLIB's supported types, an exception will be thrown.
    * \param pMemLoc pointer to beginning of memory block
    * \param nSize size of the memory block in bytes
    * \param imageType (optional) pointer to image format object
    */
   void dlLoadImage(const void* pMemLoc, const int nSize, 
                    DLBaseImage* imageType = NULL);

   /** 
    * Returns the number of pages contained in an image file
    * stored in a memory block.
    * It makes use of the DLImageFactory to
    * determine the image type and  then calls the dlGetTotalPages() function
    * for the particular image object.
    * \param pMemLoc pointer to beginning of memory block
    * \param nSize size of memory buffer in bytes
    * \return the number of pages in the image 
    */
   static int dlGetTotalPages(void* pMemLoc, int nSize);

   /** 
    * Returns the number of pages contained in an image file.
    * It make use of the DLImageFactory to
    * determine the image type and then calls the dlGetTotalPages() function
    * for the particular image object.
    * \param filename filename of image file
    * \return the number of pages in the image 
    */
   static int dlGetTotalPages(const char* filename);

   /**
    * Loads a particular page of a multiple page image file.
    * \param filename name of image file to be opened
    * \param pageNumber number of page to be opened
    * \return image of page
    */
   static DLImage dlLoadImageAtPage(const char* filename, int pageNumber) ;

   /**
    * Loads a particular page of a multiple page image file stored in a memory buffer.
    * \param pMemLoc pointer to beginning of memory Image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return image of page
    */
   static DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber) ;


   /**
    * Save an image.
    *
    * DOCLIB will determine the image format to save to by the extension of \a filename.
    * To override this choice, or if the filename does not have a valid extension,
    * you may indicate the image format to use by passing in a pointer to one
    * of the image format objects (see DLBaseImage). Example:
    * \code
    * DLImage i("test.jpg");
    * // ... processing
    * i.dlSaveImage("out.jpg"); // save as JPEG automatically
    * i.dlSaveImage("output.file", DLTIFFImage::createImageReader()); // save as TIFF
    * i.dlSaveImage("output.file", dlTiffProcessor); // save as TIFF
    * \endcode
    * \param filename full path on disk to save file (e.g. <tt>"/home/joe/tree.tif"</tt>)
    * \param imageType (optional) pointer to image format object
    * \throw DL_NULL_POINTER_EXCEPTION when data is \c NULL
    */
   void dlSaveImage(const char* filename, DLBaseImage* imageType = NULL) const;

   /**
    * Save an image to memory.
    *
    * You must indicate the image format to use by passing in a pointer to one
    * of the image format objects (see DLBaseImage). Example:
    * \code
    * DLImage i("test.jpg");
    * DL_BYTE *buff;
    * int size;
    * // ... processing
    * buff = (DL_BYTE *) i.dlSaveImageToMem(size, DLTIFFImage::createImageReader()); // save as TIFF
    * buff = (DL_BYTE *) i.dlSaveImageToMem(size, dlTiffProcessor); // save as TIFF
    * \endcode
    * \param[out] nSize this is where the buffer length will be returned)
    * \param[in] imageType pointer to image format object
    * \throw DL_NOT_SUPPORTED_EXCEPTION Not all <i>Image Types</i> support writing to memory.  
    * Currently supported types are: <i>TIFF, PBM, BMP, JPG</i>
    * \return pointer to memory buffer in which the image was saved
    * \throw DL_NULL_POINTER_EXCEPTION when data is \c NULL
    */
   void* dlSaveImageToMem(int& nSize, DLBaseImage* imageType) const;


   /**
    * \return DLImageHead object, containing the image header information.
    */
   DLImageHead dlGetImageHead()  const;

   /**
    * Get the image width 
    * \return image width
    */
   int	dlGetWidth()  const  { return imageHead.width;};

   /**
    * Get the image height
    * \return image height
    */
   int	dlGetHeight()  const  { return imageHead.height;};

   /**
    * Get the number of image data channels; usually 3 for 24-bit color
    * RGB images, and 1 for binary, grayscale or palettized images.
    * \return image channels
    */
   int	dlGetChannels()  const  { return imageHead.channels;};

   /**
    * Get the number of bits per image pixel; can be one of:
    * <UL>
    * <LI> DLImageHead::DL_BITDEPTH_1
    * <LI> DLImageHead::DL_BITDEPTH_8
    * <LI> DLImageHead::DL_BITDEPTH_24
    * </ul>
    * \return image bit depth
    */
   DLImageHead::DLBitDepths dlGetDepth()    const  { return imageHead.depth;};

   /**
    * Get the image color model; can be one of:
    * <UL>
    * <LI> DLColor::DL_BINARY
    * <LI> DLColor::DL_GRAY
    * <LI> DLColor::DL_RGBCOLOR
    * <LI> DLColor::DL_YUVCOLOR
    * <LI> DLColor::DL_YCCCOLOR
    * </ul>
    * \return image  color model
    */
   DLColor::DLColorTypes dlGetColorModel()  const  { return imageHead.colorModel;};

   /**
    * Get data order: not currently used.
    * \return  data order
    */
   int dlGetDataOrder( )  const  { return imageHead.dataOrder;};

   /**
    * Get the image row size: the number of bytes
    * needed to store the raw data of a single row of this image.
    * (Image data in DOCLIB is stored in row-major format.)
    * \return  row size
    */
   int	dlGetRowSize()    const  { return imageHead.rowSize;};

   /**
    * Get data alignment.  Can be
	* <UL>
	* <LI>DLImageHead::DL_UNKNOWN_ALIGN
	* <LI>DLImageHead::DL_1BYTE_ALIGN
	* <LI>DLImageHead::DL_4BYTE_ALIGN
	* </UL>
    * \return data alignment
    */
   DLImageHead::DLDataAlignments dlGetDataAlignment()  const  { return imageHead.dataAlignment;};

   /**
    * Get resolution (horizontal).
    * \see dlGetResolutionFlag() for determining the unit of measure.
    * \return horizontal image resolution
    */
   float dlGetHorizontalResolution()   const  { return imageHead.horizontalResolution;};

   /**
    * Get resolution  (vertical) 
    * \see dlGetResolutionFlag() for determining the unit of measure.
    * \return vertical image resolution
    */
   float dlGetVerticalResolution()   const  { return imageHead.verticalResolution;};

   /**
    * Get image resolution flag.
    * Upon opening an image of any format containing resolution information,
    * DOCLIB automatically converts the resolution to dots-per-inch
    * (or unitless, if the image does not indicate the units used).
    * \return resolution flag:
    * \arg \c true resolution in dots-per-inch (dpi)
    * \arg \c false unitless
	* \deprecated Use DLImage::dlGetResolutionUnits() instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   bool dlGetResolutionFlag () const { 
	   return (imageHead.resolutionUnits == DLImageHead::DL_RESOLUTION_DPI); };

   /**
    * Get image resolution flag.
    * Upon opening an image of any format containing resolution information,
    * DOCLIB automatically converts the resolution to dots-per-inch
    * (or unitless, if the image does not indicate the units used).
    * \return resolution untis DLImageHead::DLResolutionUnits:
    * \arg DLImageHead::DL_RESOLUTION_DPI
    * \arg DLImageHead::DL_RESOLUTION_UNITLESS
	*/
   DLImageHead::DLResolutionUnits dlGetResolutionUnits () const { return imageHead.resolutionUnits; };

   /**
    * Set image resolution (horizontal).  
    * \see dlSetResolutionFlag() for setting the resolution units of measure.
    * \param resolution image resolution
    */
   void dlSetHorizontalResolution (int resolution) { imageHead.horizontalResolution = (float)resolution;};

   /**
    * Set image resolution (vertical).
    * \see dlSetResolutionFlag() for setting the resolution units of measure.
    * \param resolution image resolution
    */
   void dlSetVerticalResolution (int resolution) { imageHead.verticalResolution = (float)resolution;};

   /**
    * Set image resolution (horizontal).
    * \see dlSetResolutionFlag() for setting the resolution units of measure.
    * \param resolution image resolution
    */
   void dlSetHorizontalResolution (float resolution) { imageHead.horizontalResolution = resolution;};

   /**
    * Set image resolution (vertical).
    * \see dlSetResolutionFlag() for setting the resolution units of measure.
    * \param resolution image resolution
    */
   void dlSetVerticalResolution (float resolution) { imageHead.verticalResolution = resolution;};

   /**
    * Set image resolution units of measure. 
    * \note Changing the resolution units of measure 
    * will not automatically convert the previously-stored resolution values.
    * \param resflag resolution flag
    * \li \c true = dpi
    * \li \c false = unitless
	* \deprecated Use DLImage::dlSetResolutionUnits() instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   void dlSetResolutionFlag (bool resflag) { imageHead.resolutionFlag = resflag; };
   

   /**
    * Set image resolution units of measure. 
    * \note Changing the resolution units of measure 
    * will not automatically convert the previously-stored resolution values.
	* \param resunits resolution units
    * \arg DLImageHead::DL_RESOLUTION_DPI
    * \arg DLImageHead::DL_RESOLUTION_UNITLESS
	*/
   void dlSetResolutionUnits (DLImageHead::DLResolutionUnits resunits) 
   { imageHead.resolutionUnits = resunits; };

   /**
    * Get image statistics, such as mean and standard deviation
    * of pixel values.
    * \return DLImageStats object containing the statistics.
    */
   DLImageStats dlGetStats() const;

   /**
    * Get a pointer to the raw image data. 
    * When writing a new DOCLIB image processing application, this is the 
    * most efficient way to access and modify the internal pixel data.
    * See also dlGetPixelValue() for an easier, but possibly less efficient
    * way of accessing a pixel's value.
    * 
    * The DOCLIB data layout is row-major; all the data for a single
    * row of the image is in a contiguous memory block, followed by the data
    * for the next row. Use the functions dlGetDepth(), dlGetColorModel()
    * and dlGetRowSize() to determine the storage format and storage size for each pixel as 
    * well as the number of bytes required for each row of the image.
    * 
    * Binary images will be stored one bit per pixel, with the leftmost pixel
    * occupying the most significant bit (MSB) and the rightmost pixel
    * occupying the least significant bit (LSB).
    * A mask may be used * to access individual pixel values this way. This statement:
    * \code
    * DL_BYTE getmask[8] = {128,64,32,16,8,4,2,1};
    * \endcode
    * sets up an array of masks; each entry in the array contains a mask with 
    * a 1 in one bit position and 0's in the other seven. <tt>mask[0]</tt> is used
    * to access the leftmost bit and <tt>mask[7]</tt> is used to access the rightmost bit.
    * To get a pixel value, AND the byte with the appropriate mask. If the result
    * is zero, then the pixel was off (black). If the result is non-zero, the pixel
    * was on (white). 
    * \code
    * DL_BYTE result = imageData[k] & mask[2]; // check third pixel
    * \endcode
    * To set a pixel to 0 (black), AND the byte of image data with the 
    * one's complement (inverse) of the appropriate mask.
    * \code
    * imageData[k] &= ~(mask[2]);
    * \endcode
    * To set a pixel to 1 (white), OR the byte of image data with the
    * appropriate mask.
    * \code
    * imageData[k] |= mask[2];
    * \endcode
    * If the width of a binary image is not divisible by 8, the last byte in a row's
    * data will contain pixel data in the most significant bits. The next row will
    * begin with a new byte; that is, rows only change on byte boundaries.
    *
    * Grayscale images are stored one byte per pixel, with 0x00 as black, and 0xFF (255)
    * as white.
    *
    * RGB color images are stored 3 consecutive bytes per pixel, in single plane format.
    * (As opposed to separate plane format; if an image file is stored in separate
    * plane format, DOCLIB converts the image to single-plane in its internal data.)
    * The first byte is the red value, the second byte is the green value, and the
    * third byte is the blue value.
    *
    * Be mindful of changing the returned data as it will also change the value
    * of the data that is a part of DLImage. 
    * If you don't want to modify the data, consider using dlGetSafeImageData()
    *  which will return a copy of the data.
    * \return image data
    */
   DL_BYTE*  dlGetImageData()  const  { return imageData;};

   /**
    * Get the raw image data. Returns a copy of the raw data to ensure that the contents in DLImage is not 
    * modified. Note, however, the impact using this function may have on performance. For time critical applications
    * consider developing using this function and using the dlGetImageData function before release.
    * See dlGetImageData() for a descripition of the DOCLIB internal data format.
    * \return image data
    * \throw DL_NULL_POINTER_EXCEPTION When data is \c NULL
    * \throw DL_MEMORY_EXCEPTION When unable to allocate enough memory for data buffer
    */
   DL_BYTE * dlGetSafeImageData()  const; 

   /**
    * Extract a region of the image (crop).
    * \param x column (<i>x</i>-coordinate)  of upper-left point of region to extract
    * \param y row (<i>y</i>-coordinate)  of upper-left point of region to extract
    * \param width width of region to extract
    * \param height height of region to extract
    * \return extracted sub-image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_WRONG_FORMAT_EXCEPTION When the subimage is out of bounds
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1, 8 or 24 bits.
    */
   DLImage dlSubImage(int x,  int y, int width, int height)  const;

   /**
    * Extract a region of the image (crop).
    * \param cropzone DLZone with stats (origin, width, height) of region to extract
    * \return extracted sub-image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_WRONG_FORMAT_EXCEPTION When the subimage is out of bounds
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1, 8 or 24 bits.
    */
   DLImage dlSubImage(const DLZone & cropzone) const;

   /**
    * Extract a region of the image (crop), perserving only those connected components
    * with an mass greater than or equal to the maximum mass * threshold
    * \param x column (<i>x</i>-coordinate)  of upper-left point of region to extract
    * \param y row (<i>y</i>-coordinate)  of upper-left point of region to extract
    * \param width width of region to extract
    * \param height height of region to extract
    * \param threshold threshold percentage whereby components less than threshold * max_mass will be filtered.
    * \return extracted sub-image
    */
   DLImage dlSubImageMaxMassCC(int x,  int y, int width, int height, float threshold = 1.0)  const;

   /**
    * Extract a region of the image (crop), perserving only those connected components
    * with an density greater than or equal to the maximum density * threshold
    * \param x column (<i>x</i>-coordinate)  of upper-left point of region to extract
    * \param y row (<i>y</i>-coordinate)  of upper-left point of region to extract
    * \param width width of region to extract
    * \param height height of region to extract
    * \param threshold threshold percentage whereby components less than threshold * max_density will be filtered.
    * \return extracted sub-image
    */
   DLImage dlSubImageMaxDensityCC(int x,  int y, int width, int height, float threshold = 1.0)  const;

   /**
    * Extract a region of the image (crop), perserving only those connected components
    * with a Height greater than or equal to the maximum Height * threshold
    * \param x column (<i>x</i>-coordinate)  of upper-left point of region to extract
    * \param y row (<i>y</i>-coordinate)  of upper-left point of region to extract
    * \param width width of region to extract
    * \param height height of region to extract
    * \param threshold threshold percentage whereby components less than threshold * max_Height will be filtered.
    * \return extracted sub-image
    */
   DLImage dlSubImageMaxHeightCC(int x,  int y, int width, int height, float threshold = 1.0)  const;

   /**
    * Extract a region of the image (crop), perserving only those connected components
    * with a Width greater than or equal to the maximum Width * threshold
    * \param x column (<i>x</i>-coordinate)  of upper-left point of region to extract
    * \param y row (<i>y</i>-coordinate)  of upper-left point of region to extract
    * \param width width of region to extract
    * \param height height of region to extract
    * \param threshold threshold percentage whereby components less than threshold * max_Width will be filtered.
    * \return extracted sub-image
    */
   DLImage dlSubImageMaxWidthCC(int x,  int y, int width, int height, float threshold = 1.0)  const;

   /**
    * Set a pixel in the image to a given color
    * \param x column (<i>x</i>-coordinate)  of point
    * \param y row (<i>y</i>-coordinate)  of point
    * \param r red value
    * \param g green value
    * \param b blue value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Color Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlSetPixel (int x, int y,  DL_BYTE r, DL_BYTE g, DL_BYTE b);

   /**
    * Set a pixel in an image to a given color.
    * \param p DLPoint containing x,y coordinates
    * \param r red value
    * \param g green value
    * \param b blue value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Color Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlSetPixel (const DLPoint &p, DL_BYTE r, DL_BYTE g, DL_BYTE b)
      { dlSetPixel (p.x,p.y,r,g,b); };

   /**
    * Get the value of a pixel in an RGB color image.
    * \param[in] x column (<i>x</i>-coordinate)  of point
    * \param[in] y row (<i>y</i>-coordinate)  of point
    * \param[out] r red value
    * \param[out] g green value
    * \param[out] b blue value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Color Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlGetPixel (int x, int y,  DL_BYTE &r, DL_BYTE &g, DL_BYTE &b) const;

   /**
    * Get the value of a pixel in an RGB color image.
    * \param[in] p DLPoint containing x,y coordinates
    * \param[out] r red value
    * \param[out] g green value
    * \param[out] b blue value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Color Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlGetPixel (const DLPoint &p,  DL_BYTE &r, DL_BYTE &g, DL_BYTE &b) const
      { dlGetPixel (p.x,p.y,r,g,b); };

   /**
    * Set a pixel in an image to a given grayscale value.
    * \param[in] x column (<i>x</i>-coordinate)  of point
    * \param[in] y row (<i>y</i>-coordinate)  of point
    * \param i grayscale value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Gray or BW Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlSetPixel (int x, int y, DL_BYTE i);

   /**
    * Set a pixel in an image to a given grayscale value.
    * \param p DLPoint containing x,y coordinates
    * \param i grayscale value
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Gray or BW Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   void dlSetPixel (const DLPoint &p, DL_BYTE i)
      { dlSetPixel (p.x,p.y,i); };

   /**
    * Gets a pixel value as a DLColor object.
    * \param p pixel location
    * \return pixel value
    */
   DLColor dlGetPixel (const DLPoint & p) const
   { return dlGetPixel(p.x, p.y); }

   /**
    * Gets a pixel value as a DLColor object.
    * \param x column (<i>x</i>-coordinate)  of pixel
    * \param y row (<i>y</i>-coordinate)  of pixel
    * \return pixel value
    */
   DLColor dlGetPixel (int x, int y) const;

   /** 
    * Sets a pixel to the given color.
    * \param p pixel location
    * \param c desired color
    */
   void dlSetPixel (const DLPoint & p, const DLColor & c)
   { dlSetPixel(p.x, p.y, c); }

   /**
    * Sets a pixel to the given color.
    * \param x column (<i>x</i>-coordinate)  of pixel
    * \param y row (<i>y</i>-coordinate)  of pixel
    * \param c desired color
    */
   void dlSetPixel (int x, int y, const DLColor & c);

   /**
    * Get the pixel value of a binary or grayscale image.
    * For a binary image, this will be either ::DL_WHITE or ::DL_BLACK.
    * For a grayscale image, this will range from 0 to 255, with 0 being black
    * and 255 being white.
    * \param x column, or <i>x</i>-coordinate of requested pixel
    * \param y row, or <i>y</i>-coordinate of requested pixel
    * \return DL_BYTE value of requested pixel.
    * \throw DLException::DL_WRONG_FORMAT_EXCEPTION When Image is not a Gray or BW Image
    * \throw DLException::DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DLException::DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   DL_BYTE dlGetPixelValue (int x, int y) const;

   /**
    * Get the pixel value of a binary or grayscale image.
    * For a binary image, this will be either ::DL_WHITE or ::DL_BLACK.
    * For a grayscale image, this will range from 0 to 255, with 0 being black
    * and 255 being white.
    * \param p DLPoint containing x,y coordinates of requested pixel
    * \return DL_BYTE value of requested pixel.
    * \throw DL_WRONG_FORMAT_EXCEPTION When Image is not a Gray or BW Image
    * \throw DL_WRONG_FORMAT_EXCEPTION When <i>(x,y)</i> is out of bounds
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   DL_BYTE dlGetPixelValue (const DLPoint &p) const
      { return dlGetPixelValue(p.x,p.y); };

   /**
    * Copy image
    * \return copied image
    */
   DLImage dlCopyImage() const;

   /**
    * Checks to see if a grayscale image has only black or white values
    * \return \c true if a BW image; \c false if intermediate grayscale
    * values are present
    */
   bool dlIsGrayScaleBW() const;

   /**
    * Check if a point is inside this image.
    * \param x column of point
    * \param y row of point   
    * \return \c true if it is
    */
   bool dlIsInside(const int x, const int y) const;

   /**
    * Check if a point is inside this image.
    * \param p DLPoint containing pixel x,y coordinates
    * \return \c true if it is
    */
   bool dlIsInside(const DLPoint & p) const
      { return dlIsInside(p.x,p.y); } ;

   /**
    * Paste an image onto the current image. This function
    * returns a new image with the pasted region; to write to the current
    * image, see dlPasteInImage().
    * \param subImage source image to be pasted
    * \param xcoor column (<i>x</i>-coordinate) of destination image to paste to
    * \param ycoor row (<i>y</i>-coordinate) of destination image to paste to
    * \return DLImage new image with pasted region
    */
   DLImage dlPasteImage(const DLImage& subImage, int xcoor, int ycoor) const;

   /**
    * Paste an image onto the current image. This function
    * returns a new image with the pasted region; to write to the current
    * image, see dlPasteInImage().
    * \param subImage source image to be pasted
    * \param p DLPoint containing pixel x,y coordinates of destination image to paste to
    * \return DLImage new image with pasted region
    */
   DLImage dlPasteImage(const DLImage& subImage, const DLPoint &p) const
      { return dlPasteImage(subImage,p.x,p.y); };

   /**
    * Paste an image onto the current image; will modify the current 
    * image data.  There will be no return value since the image
    * data is being modified. See also dlPasteImage().
    *
    * \param subImage source mage to be pasted
    * \param xcoor column (<i>x</i>-coordinate) of destination image to paste to
    * \param ycoor row (<i>y</i>-coordinate) of destination image to paste to
    */
   void dlPasteInImage( const DLImage& subImage, int xcoor, int ycoor);

   /**
    * Paste an image onto the current image; will modify the current 
    * image data.  There will be no return value since the image
    * data is being modified. See also dlPasteImage().
    *
    * \param subImage source mage to be pasted
    * \param p DLPoint containing pixel x,y coordinates of destination image to paste to
    */
   void dlPasteInImage( const DLImage& subImage, const DLPoint &p)
      { dlPasteInImage(subImage,p.x,p.y); };

   /**
    * Add an image into the current image.  <b>Note:</b> At this time,
    * pixel values that exceed the maximum will overflow and not be
    * capped at 255.
    * \param aImg  image to be added
    * \param xcoor column (<i>x</i>-coordinate) of destination image
    * \param ycoor row (<i>y</i>-coordinate) of destination image
    * \return DLImage image with added region
    */
   DLImage dlAddImage(const DLImage& aImg, int xcoor, int ycoor) const;

   /**
    * Add an image into the current image.  <b>Note:</b> At this time,
    * pixel values that exceed the maximum will overflow and not be
    * capped at 255.
    * \param aImg  image to be added
    * \param p DLPoint containing pixel x,y coordinates of destination image
    * \return DLImage image with added region
    */
   DLImage dlAddImage(const DLImage& aImg, const DLPoint &p) const
      { return dlAddImage (aImg,p.x,p.y); };

   /**
    * Subtract an image from the current image
    * \param aImg  image to be subtracted
    * \param xcoor column (<i>x</i>-coordinate) of destination image
    * \param ycoor row (<i>y</i>-coordinate) of destination image
    * \return DLImage image with subtracted region
    */
   DLImage dlSubtractImage(const DLImage& aImg, int xcoor, int ycoor) const;

   /**
    * Subtract an image from the current image
    * \param aImg  image to be subtracted
    * \param p DLPoint containing pixel x,y coordinates of destination image
    * \return DLImage image with subtracted region
    */
   DLImage dlSubtractImage(const DLImage& aImg, const DLPoint &p) const
      { return dlSubtractImage (aImg,p.x,p.y); };

   /**
    * Scale image to specified height and width
    * \param width scaled image width
    * \param height scaled image height
    * \return scaled image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_NOT_SUPPORTED_EXCEPTION When \c width or \c height are less than 0
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1, 8 or 24 bits.
    */
   DLImage dlResizeImage(int width, int height) const;

   /**
    * Scale image by a factor
    * \param factor scale factor (e.g. 1.0 keeps the image the same, 0.5 
    * makes the new image half the size; 2.0 doubles the size of the image)
    * \return scaled image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_NOT_SUPPORTED_EXCEPTION When \c factor is less than 0
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1, 8 or 24 bits.
    */
   DLImage dlResizeImage(double factor) const;

   /**
    * Rotate image
    * \param angle angle to rotate image by (in degrees). The sign of angle controls direction of rotation:  
    * positive angle - clockwise, negative - counterclockwise
    * \return rotated image in its original bits-per-pixel representation
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1, 8 or 24 bits.
    */
   DLImage dlRotateImage(double angle) const;

   /**
    * Flip image
    * \param direction direction to flip the image: can be one of
    * \arg DLImage::DL_HORIZONTAL
    * \arg DLImage::DL_VERTICAL
    * \return flipped image
    */
   DLImage dlFlipImage(DLFlipDirection direction) const;

   /**
    * Reverse Image
    * \return DLImage of reversed image
    */
   DLImage dlReverseImage() const;

   /**
    * Get a one channel image from a color image. The image returned uses
    * one byte per pixel and behaves as a grayscale image.
    * \param ch number of channel to be returned:
    * \arg DLImage::DL_RED_CHANNEL
    * \arg DLImage::DL_GREEN_CHANNEL
    * \arg DLImage::DL_BLUE_CHANNEL
    * \return channel image
    */
   DLImage dlGetChannelImage (DLImage::DLColorChannels ch) const;

   /**
    * Get a one channel image from a color image. The image returned uses
    * one byte per pixel and behaves as a grayscale image.
    * \param ch number of channel to be returned:
    * \arg 0 red channel
    * \arg 1 green channel
    * \arg 2 blue channel
    * \return channel image
	* \deprecated Use DLImage::dlGetChannelImage (DLImage::DLColorChannels ch) instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   DLImage dlGetChannelImage (int ch) const;

   /**
    * Get an RGB color image from three one-byte-per-pixel channel images.
    * Example:
    * \code
    * DLImage redim, blueim, greenim;
    * // ... processing
    * DLImage colorimage = DLImage::dlMergeChannelImage(&redim, &greenim, &blueim);
    * \endcode
    * \param[in] im1 red channel image
    * \param[in] im2 green channel image
    * \param[in] im3 blue channel image
    * \return full color image
    */
   static DLImage	dlMergeChannelImage (DLImage * im1, DLImage * im2, DLImage * im3);

   /**
    * Convert DLImage to the YCrCb colorspace from RGB.
    * The Y (brightness), Cr (red chrominance), and Cb (blue chrominance) values 
    * are calculated from the R, G, and B values of a pixel as follows:
    * \f[
    * \left[ \begin{array}{l} Y \\ Cr \\ Cb \end{array} \right] = 
    * \left[ \begin{array}{rrrr} 0.257 & 0.504 & 0.098 & 16 \\
    0.439 &-0.368 &-0.071 & 128 \\
    -0.148 &-0.291 & 0.439 & 128 \end{array} \right]
    * \left[ \begin{array}{l} R \\ G \\ B \\ 1 \end{array} \right] \f]
    * Note that the header of the image returned (see DLImageHead::colorModel) 
    * will still indicate the original image's colorspace, and will not be modified
    * to reflect that the updated pixel values are in the YCrCb colorspace,
    * nor will it be checked to verify that
    * the original image did in fact use the RGB colorspace.
    * <br><br>This function is in <b>Stage One</b> of the documented <i>Three Stage Deprecation Process</i>
    * \return YCrCb image
    * \deprecated Use DLImage::dlConvertRGBToYCrCb() instead.
	* <br>This is in <b>Phase Two</b> of the <i>Documented Deprecation Process</i>
    */
   DLImage dlConvertToYcrCb();

   /**
    * Convert DLImage to the YCrCb colorspace from RGB.
    * The Y (brightness), Cr (red chrominance), and Cb (blue chrominance) values 
    * are calculated from the R, G, and B values of a pixel as follows:
    * \f[
    * \left[ \begin{array}{l} Y \\ Cr \\ Cb \end{array} \right] = 
    * \left[ \begin{array}{rrrr} 0.257 & 0.504 & 0.098 & 16 \\
    0.439 &-0.368 &-0.071 & 128 \\
    -0.148 &-0.291 & 0.439 & 128 \end{array} \right]
    * \left[ \begin{array}{l} R \\ G \\ B \\ 1 \end{array} \right] \f]
    * Note that the header of the image returned (see DLImageHead::colorModel) 
    * will still indicate the original image's colorspace, and will not be modified
    * to reflect that the updated pixel values are in the YCrCb colorspace,
    * nor will it be checked to verify that
    * the original image did in fact use the RGB colorspace.
    * \return YCrCb image
    */
   DLImage dlConvertRGBToYCrCb() const;

   /**
    * Convert DLImage to the RGB colorspace from YCrCb, using the following formulas:
    * \f{eqnarray*}
    * R &=& 1.164(Y - 16)+1.596(Cr - 128) \\
    * G &=& 1.164(Y - 16)-0.813(Cr - 128)-0.391(Cb - 128) \\
    * B &=& 1.164(Y - 16)+2.018(Cb - 128)
    * \f}
    * Note that the header of the image returned (see DLImageHead::colorModel) 
    * will still indicate the original image's colorspace, and will not be modified
    * to reflect that the updated pixel values are in the RGB colorspace,
    * nor will it be checked to verify that
    * the original image did in fact use the YCrCb colorspace.
    * <br><br>This function is in <b>Stage One</b> of the documented <i>Three Stage Deprecation Process</i>.
    * \return DLImage RGB image
    * \deprecated Use DLImage::dlConvertYCrCbToRGB() instead.
	* <br>This is in <b>Phase Two</b> of the <i>Documented Deprecation Process</i>
    */
   DLImage dlConvertYcrCbToRGB() ;

   /**
    * Convert DLImage to the RGB colorspace from YCrCb, using the following formulas:
    * \f{eqnarray*}
    * R &=& 1.164(Y - 16)+1.596(Cr - 128) \\
    * G &=& 1.164(Y - 16)-0.813(Cr - 128)-0.391(Cb - 128) \\
    * B &=& 1.164(Y - 16)+2.018(Cb - 128)
    * \f}
    * Note that the header of the image returned (see DLImageHead::colorModel) 
    * will still indicate the original image's colorspace, and will not be modified
    * to reflect that the updated pixel values are in the RGB colorspace,
    * nor will it be checked to verify that
    * the original image did in fact use the YCrCb colorspace.
    * \return DLImage RGB image
    */
   DLImage dlConvertYCrCbToRGB() const;


   /**
    * Convert DLImage from the RGB colorspace to the HSV colorspace.
    * The H (hue), S (saturation) and V (value / brightness) are calculated for each
    * pixel using the following formulas:
    * \f{eqnarray*}
    * H &=& 60 \times \left\{ \begin{array}{ll} \frac{G-B}{\max - \min}, & \max = R \\
    2 + \frac{B-R}{\max - \min} + 120, & \max = G \\
    4 + \frac{R-G}{\max - \min} + 240, & \max = B
    \end{array} \right. \\
    * S &=& \frac{\max - \min}{\max} \\
    * V &=& \max
    * \f} 
    * where \f$\max = \max(R,G,B)\f$ and \f$\min = \min(R,G,B)\f$.
    *
    * Note that the header of the image returned (see DLImageHead::colorModel) 
    * will still indicate the original image's colorspace, and will not be modified
    * to reflect that the updated pixel values are in the HSV colorspace,
    * nor will it be checked to verify that
    * the original image did in fact use the RGB colorspace.
    * \return HSV image
    */
   DLImage dlConvertRGBToHSV() const;

   /**
    * Convert DLImage to the RGB colorspace from the HSV colorspace, using the
    * following calculations:
    * \f{eqnarray*}
    * H_i &=& \left\lfloor \frac{H}{60} \right\rfloor \\
    * f &=& \frac{H}{60} - H_i \\
    * p &=& V(1-S) \\
    * q &=& V(1-fS) \\
    * t &=& V(1-(1-f)S)
    * \f}
    * \f{eqnarray*}
    * \mbox{if}\: H_i = 0 &\mbox{then}& R=V, G=t, B=p \\
    * \mbox{if}\: H_i = 1 &\mbox{then}& R=q, G=V, B=p \\
    * \mbox{if}\: H_i = 2 &\mbox{then}& R=p, G=V, B=t \\
    * \mbox{if}\: H_i = 3 &\mbox{then}& R=p, G=q, B=V \\
    * \mbox{if}\: H_i = 4 &\mbox{then}& R=t, G=p, B=V \\
    * \mbox{if}\: H_i = 5 &\mbox{then}& R=B, G=p, B=q \\
    * \f}
    * \return RGB image
    */
   DLImage dlConvertHSVToRGB() const; 

   /**
    * Convert DLImage to the BGR colorspace from the RGB colorspace.
    *
    * \note The header of the image returned (see
    * DLImageHead::colorModel) will still indicate the original
    * image's colorspace, and will not be modified to reflect that the
    * updated pixel values are in the BGR colorspace, nor will it be
    * checked to verify that the original image did inface use the
    * RGB colorspace.
    * \return BGR image
    */
   DLImage dlConvertRGBToBGR() const;

   /**
    * Convert DLImage to the RGB colorspace from the BGR colorspace.
    *
    * \note The header of the image returned (see
    * DLImageHead::colorModel) will still indicate the original
    * image's colorspace, and will not be modified to reflect that the
    * updated pixel values are in the RGB colorspace, nor will it be
    * checked to verify that the original image did inface use the
    * BGR colorspace.
    * \return BGR image
    */
   DLImage dlConvertBGRToRGB() const;

   /**
    * Calculate the centroid of the image.
    * \return centroid
    */
   DLPoint dlCentroid() const;

   //don't want these public functions to show up in Doxygen
   /**
    *  Set Check if original image is corrupted
    * \param isCorrupted true: to indicate the image is corrupted.
    *                    false: to indicate the image is not corrupted (Default). 
    */
   void setIsImageCorrupted(bool isCorrupted);

   private:
   
   /**
    * dlLoad function creates an empty image with width, height, and depth header data. Additionally, 
    * memory can be set aside for imageData as indicated by the data flag.
    * \param w width of image
    * \param h height of image
    * \param depth data of image
    * \param data boolean determining whether or not memory will be allocated for imageData
	* \deprecated Use dlLoadImage(int w, int h, DLImageHead::DLBitDepths depth, bool data) instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   void dlLoadImage(int w, int h, int depth, bool data);
 
   /**
    * dlLoad function creates an empty image with width, height, and depth header data. Additionally, 
    * memory can be set aside for imageData as indicated by the data flag.
    * \param w width of image
    * \param h height of image
    * \param depth data of image
    * \param data boolean determining whether or not memory will be allocated for imageData
    */
   void dlLoadImage(int w, int h, DLImageHead::DLBitDepths depth, bool data);

   /**
    * Resize bit (BW) image to width/height input parameters
    * \param width width of image
    * \param height height of image
    * \return DLImage resized image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_NOT_SUPPORTED_EXCEPTION When width or height are less than 0
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 1-bit BW Image
    */
   DLImage dlResizeBitImage(int width, int height) const;

   /**
    * Resize byte image to width/height input parameters
    * \param width width of image
    * \param height height of image
    * \return DLImage resized image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_NOT_SUPPORTED_EXCEPTION When width or height are less than 0
    * \throw DL_NOT_SUPPORTED_EXCEPTION When image is not 8-bit Grayscale Image
    */
   DLImage dlResizeByteImage(int width, int height) const;

   /**
    * Rotate a bit (BW) image
    * \param angle double to rotate the bit image by (in degrees): positive angle - clockwise, negative - counterclockwise
    * \return DLImage rotated bit image
    * \throw DL_NOT_SUPPORTED_EXCEPTION When Image is not a BW Image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   DLImage dlRotateBitImage(double angle)  const;

   /**
    * Rotate a byte image
    * \param angle double to rotate the byte image by (in degrees): positive angle - clockwise, negative - counterclockwise
    * \return DLImage rotated byte image
    * \throw DL_NOT_SUPPORTED_EXCEPTION When Image is not a Grayscale Image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    */
   DLImage dlRotateByteImage(double angle)  const;

   /**
    * Sub byte of an image
    * \param x x values of sub image
    * \param y y values of sub image
    * \param width of sub image
    * \param height height  sub image
    * \throw DL_NULL_POINTER_EXCEPTION When image data is \c NULL
    * \throw DL_WRONG_FORMAT_EXCEPTION When image is not 8-Bit Grayscale
    */
   DLImage dlSubByteImage(int x,  int y, int width, int height) const;

   /**
    * Take the inverse of the image
    * \return DLImage inverted image
    */
   DLImage dlReverseByteImage() const;

   /**
    * Flip the image
    * \param direction the direction to flip
    * \return DLImage inverted image
    */
   DLImage dlFlipByteImage(DLFlipDirection direction) const;

   /**
    * Take the inverse fo the image
    * \param subImage image to paste from
    * \param xcoor x coor
    * \param ycoor y coor
    * \return DLImage new image after pasting
    */
   DLImage dlPasteByteImage(const DLImage &subImage, int xcoor, int ycoor) const;

   /**
    * Helper function to paste an image onto the current DLImage.
    * This function will modify the current image data.
    *
    * \param subImage image to paste from
    * \param xcoor x coor
    * \param ycoor y coor
    */
   void dlPasteInByteImage(const DLImage &subImage, int xcoor, int ycoor) ;

   /**
    * Normalize the pixels of a grayscale image based on max and min
    * Note that max and min are determined for all pixels even in a color image
    * \return DLImage Normalized image
    */
   DLImage dlNormalizeGrayscaleImage();

   /**
    * Normalize the pixels of a color image based on max and min of each channel
    * \return DLImage of normalized image
    */
   DLImage dlNormalizeColorImage();

   /**
    * Normalize the pixels of the image based on max and min
    * Note that max and min are determined for all pixels even in a color image
    */
   DLImage dlNormalizeGlobalImage();

   /**
    * Return the centroid of the Grayscale DLImage 
    * \return DLPoint 
    */
   DLPoint dlCentroidGray() const;
   /**
    * Return the centroid of the Color DLImage 
    * \return DLPoint 
    */
   DLPoint dlCentroidColor() const;
   /**
    * Return the centroid of the Binary DLImage 
    * \return DLPoint 
    */
   DLPoint dlCentroidBinary() const;

   public:

   /// \name Tag List Functions
   //@{

   /**
    * Obtain an iterator to the beginning of the image tag list.
    * User-defined metadata may be stored in the DLTagList contained in the 
    * image's header. This set of functions is for setting and inspecting tags.
    * Besides typical get and set functions, and iterator interface is provided
    * for iterating over all the entries in the tag list.
    * The iterators behave the same as typical STL iterators: 
    * dlGetTagListBegin() and dlGetTagListEnd() are used to generate the iterators
    * for use in a \c for or \c while loop; the iterators can be incremented
    * or decremented using \c ++ and \c --, and can be dereferenced using the
    * * and arrow operators. Dereferencing an iterator produces a pair<string,string> 
    * where the first element is the tag key, and the second element is the tag
    * value. 
    * \code
    * DLImage i("test.jpg");
    * DLTagList::iterator ii;
    * for (ii = i.dlGetTagListBegin(); ii != i.dlGetTagListEnd(); ii++) {
    *   cout << "tag key: " << ii->first << " tag value: " << ii->second << endl;
    * }
    * \endcode
    */
   DLTagList::iterator dlGetTagListBegin() { return imageHead.imageTags.begin(); }

   /**
    * Obtain an iterator pointing just past the end of the image tag list.
    * The end iterator is typically used in a \c for loop for bounds checking.
    * Warning: actually dereferencing the end
    * iterator is undefined, and will likely cause your program to crash.
    */
   DLTagList::iterator dlGetTagListEnd() { return imageHead.imageTags.end(); }

   /**
    * Get an iterator pointing to the location of \c tagKey in the map
    * \return iterator
    */
   DLTagList::iterator dlFindTag ( string tagKey ) { return imageHead.imageTags.dlFindTag(tagKey); }

   /**
    * Set an image tag.  If a tag with the specified key already exists
    * in this image, it will not be overwritten unless the optional parameter
    * \a overwriteEnabled is set to \c true.
    * \param tagKey key of the image tag
    * \param tagValue value of the image tag
    * \param overwriteEnabled option for overwriting existing field (default is \c false )
    * \throw DL_Exception DL_UNKNOWN_TAG_EXCEPTION
    */
   void dlSetTag(string tagKey, string tagValue, bool overwriteEnabled = false);

   /**
    * Remove the tag with the specified key from the list of image tags
    * \param tagKey key of the image tag
    */
   void dlDeleteTag(string tagKey);

   /**
    * Erase all the existing image tags
    */
   void dlClearTags() { imageHead.imageTags.dlClearTags(); };

   /**
    * Check whether a document tag with the given key exists
    * \param tagKey key of the document tag
    * \return true if <i><b>a</b> tag exists</i> with that key
    */
   bool dlIsTagSet(string tagKey) const;

   /**
    * Check whether the whole image tag list is empty
    * \return \c true if the tag list is empty
    */
   bool dlIsTagListEmpty()  const { return imageHead.imageTags.dlIsTagListEmpty(); };
   /**
    * Get the field value of a specified image tag
    * \param tagKey image tag key
    */
   string dlGetTag(string tagKey) const;

   //@}

   /**
    * Get the total number of bytes in the image
    * \return the number of bytes in the image.
    */
   int dlGetImageBuffSize(){return this->dlGetRowSize() * this->dlGetHeight();};

   /**
    * Indicates if the image was considered corrupted upon loading.
    * Each image object will have its own error recovery/correction
    * algorithm when loading an image. Look at image objects for more details.
    *  \return bool 
    */
   bool isImageCorrupted();


/** \cond **/
   /**
    * Rotate a point. Used by dlRotateImage().
    * \param[in] p1 point1
    * \param[out] p2 point2
    * \param angle angle to rotate the point by (in radians): positive angle - clockwise, negative - counterclockwise
    */
   void dlRotatePoint(DLPoint *p1,  DLPoint *p2, double angle) const;

   /**
    * Return the centroid of the data buffer (Gray Scale).
    * \param w width of the image
    * \param h height of the image
    * \param data data buffer of the image
    * \return DLPoint 
    */
   static DLPoint dlCalcCentroid(int w, int h, DL_BYTE *data);
/** \endcond **/

   /**
    * Convert a given RGB triple to the HSV color space
    * \param[in] r R value
    * \param[in] g G value
    * \param[in] b B value
    * \param[out] h H value to be converted to
    * \param[out] s S value to be converted to
    * \param[out] v v value to be converted to
    */
   static void RGBtoHSV( float r, float g, float b, float *h, float *s, float *v );

   /**
    * Convert a given HSV triple to the RGB color space
    * \param[in] h H value 
    * \param[in] s S value 
    * \param[in] v v value 
    * \param[out] r R value to be converted to
    * \param[out] g G value to be converted to
    * \param[out] b B value to be converted to
    */
   static void HSVtoRGB( float *r, float *g, float *b, float h, float s, float v );

    
   /**
    *   Calculate the maximum within-image distance from centroid of a full image.
    *   \param flag Can be either:
    *     \li true to calculate the farthest image corner
    *     \li false to calculate the farthest non-white pixel
    *   \return maximum distance
    */
   double dlRmax(bool flag) const;

   private:


/** \cond **/

   double dlCalcRmax(int w, int h, DLPoint centroid, DL_BYTE *data) const;
   double dlRmaxBinary(DLPoint centroid) const;
   double dlRmaxGray(DLPoint centroid) const;
   double dlRmaxColor(DLPoint centroid) const;

   /**
    * Sets raw image data
    * \param data image data
    */
   void dlSetImageData( DL_BYTE *data);   

   // Set Image head info and data

   /**
    * Set image height
    * \param h image height
    */
   void dlSetHeight(int h)		{imageHead.height = h;};

   /**
    * Set image width
    * \param w image width
    */
   void dlSetWidth(int w)		{imageHead.width = w;};

   /**
    * Set image channels
    * \param c image channels
    */
   void dlSetChannels(int c)	{imageHead.channels = c;};

   /**
    * Set image depth
    * \param dep image depth
    */
   void dlSetDepth(DLImageHead::DLBitDepths dep)	{imageHead.depth = dep;};

   /**
    * Set image color model
    * \param cm image color model
    */
   void dlSetColorModel(DLColor::DLColorTypes cm){ imageHead.colorModel = cm;};

   /**
    * Set data alignment
    * \param align image data alignment
	* \deprecated Use dlSetDataAlignment( DLImageHead::DLDataAlignments align) instead
	* <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   void dlSetDataAlignment(int align) { 
	   
	   dlLogger.dlLog(DL_CRITICAL, "Deprecation Warning: DLImage::dlSetDataAlignment(int align)\n"
		"\tDLImage::dlSetDataAlignment(int align) has been replaced by\n\tDLImage::dlSetDataAlignment(DLImageHead::DLDataAlignments)\n"
		"\tand will be deprecated over the course of the following two releases\n"
		"\taccording to the deprecation procedure outlined by the DOCLIB Media\n\tProcessing Team.\n\n");

	   if (align != 1 && align != 4)
		   align = -1; //invalid

	   dlSetDataAlignment((DLImageHead::DLDataAlignments)align);
   }; 

   /**
    * Set data alignment
    * \param align image data alignment
    */
   void dlSetDataAlignment( DLImageHead::DLDataAlignments align) { imageHead.dataAlignment = align;}; 

   /**
    * Set row size
    * \param rs image row size
    */
   void dlSetRowSize (int rs)    {  imageHead.rowSize = rs;};

   /**
    * Set data order
    * \param data data order
    */
   void dlSetDataOrder(int data) { imageHead.dataOrder = data;};

   /**
    * Sets imageHeader information including width, height, depth, horizontal resolution, and vertical 
    * resolution
    * \param w image width
    * \param h image height
    * \param depth image depth
    * \param hres horizontal resolution
    * \param vres vertical resolution
    * \param rflag
    * \deprecated Please use dlSetImageHead(int, int, int, float, float, DLImageHead::DLResolutionUnits) instead
    */
   void dlSetImageHead(int w, int h, int depth, float hres, float vres, bool rflag);

   /**
    * Sets imageHeader information including width, height, depth, horizontal resolution, and vertical 
    * resolution
    * \param w image width
    * \param h image height
    * \param depth image depth
    * \param hres horizontal resolution
    * \param vres vertical resolution
    * \param rUnits
    * \arg DLImageHead::DL_RESOLUTION_DPI;
    * \arg DLImageHead::DL_RESOLUTION_UNITLESS;
    */
   void dlSetImageHead(int w, int h, int depth, float hres, float vres, DLImageHead::DLResolutionUnits rUnits);

   /**
    * Allocate memory for image data. If the image data memory was already
    * allocated, it will be deleted, and set to all ::DL_WHITE (0xFF).
    * \throw DL_MEMORY_EXCEPTION When unable to allocate enough memory for data buffer
    */
   void dlAllocImageData();
/** \endcond **/

   protected:
   /// Stores image header information 
   DLImageHead  imageHead;

   ///  The image data  buffer
   unsigned char* imageData;	

   /// Flag: indicates if the image was considered corrupted upon loading.
   /// Each image object will have its own error recovery/correction
   /// algorithm when loading an image. 
   /// Can be checked with isImageCorrupted().
   bool isOrgImgCorrupted;

};//end class


#endif // _DLIMAGE_H_
