#ifndef _DLLINESHAPE_H_
#define _DLLINESHAPE_H_

#include "DLShape.h"
#include "DLPoint.h"

/**
 * \brief An object to represent a line segment.
 * 
 * DLLineShape is a shape object that represents a line
 * by its two endpoints.  
 * The DLLineShape object can also be used to
 * draw the line segment it represents onto a DLImage with a particular
 * color.
 * 
 * Typical usage:
 * \code 
 * // load an image
 * DLImage myImage("myfile.tif");
 * // define the point
 * DLLineShape line(1,1, 20,40);
 * //Set the line color
 * line.dlSetLineColor(DLColor(DL_BLANCHEDALMOND));
 * //Draws the line onto a DLImage
 * line.dlDrawShape(&myImage);
 * 
 * \endcode
 */
class DLLineShape : public DLShape
{
   public:
	
   /**
    * Default DLLineShape Constructor
    */
   DLLineShape( );

   /**
    *  DLLineShape Copy  Constructor
    *  \param orig DLLineShape to be copied
    */
   DLLineShape(const DLLineShape & orig);


   /**
    *  Assignment operator
    * \param right DLLineShape to be copied
    */
   const DLLineShape & operator=(const DLLineShape & right);

   /**
    * Constructs a DLLineShape using its two endpoints.
    * \param pp1 first point in the line
    * \param pp2 second point in the line
    */
   DLLineShape(const DLPoint &pp1, const DLPoint &pp2) : DLShape()
      { p1.x = pp1.x; p1.y = pp1.y;  p2.x = pp2.x; p2.y = pp2.y; };

   /**
    * Constructs a DLLineShape using its two endpoints.
    * \param x1 column, or <i>x</i>-coordinate of first point
    * \param y1 row, or <i>y</i>-coordinate of first point
    * \param x2 column, or <i>x</i>-coordinate of second point
    * \param y2 row, or <i>y</i>-coordinate of second point
    */
   DLLineShape(int x1, int y1, int x2, int y2): DLShape()
      {p1.x = x1; p1.y = y1;  p2.x = x2; p2.y = y2; };

   /**
    * Default DLLineShape Destructor
    */
   virtual ~DLLineShape() {};


   /**
    * Draws the line shape onto a DLImage
    * \param image DLImage to draw onto, passed by pointer (see above code example)
    */
   void dlDrawShape( DLImage *image);

   /**
    * Get the first point 
    * \return DLPoint
    */
   DLPoint dlGetFirstPoint() const;

   /**
    * Set Point 1
    * \param pt first point of the line
    */
   void dlSetFirstPoint(const DLPoint &pt);
   
   /**
    * Get the second point
    * \return DLPoint
    */
   DLPoint dlGetSecondPoint() const;

   /**
    * Set Point 2
    * \param pt point2
    */
   void dlSetSecondPoint(const DLPoint &pt);

   protected:
   /// First point
   DLPoint p1;
   /// Second point
   DLPoint p2;    
};  

#endif
