#ifndef _DLPPMIMAGE_H_
#define _DLPPMIMAGE_H_

#include "DLImage.h"
#include "DLBaseImage.h"
#include "DLLogger.h"
#include "DLSingletonDestroyer.h"
/**
 * \brief BMP image I/O class.
 *
 * Implements loading and saving for the PPM image format, and is used by DLImageFactory.
 * See DLImage for the typical interface for loading and saving images. 
 */
class DLPPMImage : public DLBaseImage
{
   /// Friend to DLSingletonDestroyer object
   /// to allow the singleton object to be 
   /// destructed at the end of the application.
   template <class SINGLETON> friend class DLSingletonDestroyer;
   private:

   /**
    * Default DLPPMImage Constructor
    */
   DLPPMImage();

   ///Only instance of DLPPMImage
   static DLPPMImage* ppmImageReader;
   /// an instance of DLSingletonDestroyer to 
   /// use to destruct this singleton object
   static DLSingletonDestroyer<DLPPMImage> _destroyer;

   	/// PPM Line length
	const static int DL_PPM_MAX_LINE_LENGTH = 256;

   protected:
   /**
    * Default DLPPMImage Destructor
    */
   virtual ~DLPPMImage(){};

   public:



   /// \returns A handle to the DLPPMImage object.
   static DLPPMImage * createImageReader();


   /**
    * Load PPM Image
    * \param[in] filename name of file to be loaded
    * \param[out] image DLImage object passed by reference; will be overwritten to contain image from PPM file.
    */
   void dlLoadImage(const char* filename, DLImage& image);

   /**
    * Load PPM Image from memory
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize size of buffer in bytes
    * \return DLImage of memory buffer
    */
   DLImage dlLoadImage(const void* pMemLoc, int nSize);

   /**
    * Load PPM Image
    * \param filename name of file to be opened
    * \return DLImage of file
    */
   DLImage dlLoadImage(const char* filename);

   /**
    * Loads a particular page from a multiple page
    * image file.
    * \param filename name of file to be opened
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const char* filename, int pageNumber);

   /**
    * Loads a particular page from a mulitple page
    * image stored in a memory buffer.
    * \param pMemLoc pointer to beginning of memory image buffer to be opened
    * \param nSize size of buffer in bytes
    * \param pageNumber number of page to be opened
    * \return DLImage of particular page
    */
   DLImage dlLoadImageAtPage(const void* pMemLoc, int nSize, int pageNumber);

   /**
    * Save PPM Image
    * \param filename name of file to be saved
    * \param dlimage image to be saved
    */
   void dlSaveImage(const char* filename, const DLImage& dlimage);

   /**
    * Save PPM Image to memory
    * \param[out] nSize size of tiff file in memory
    * \param[in] dlimage DLImage*  PPM Image
    * \return pointer to the memory buffer in which the image was saved
    * \throws DL_NOT_SUPPORTED_EXCEPTION
    */
   void* dlSaveImageToMem( int& nSize, const DLImage& dlimage);

   /**
    * Determines if an image file is in the PPM format
    * and is supported for loading by DLPPMImage.
    * \param filename name of file to check
    * \return boolean: true if supported
    **/
   bool dlSupportImageFile(const char* filename);

   /**
    * Determines if an image memory buffer is in the PPM format
    * and is supported for loading by DLPPMImage.
    * \param pMemLoc pointer to beginning of buffer to check
    * \param size size of buffer in bytes
    * \return boolean: true if supported
    **/
   bool dlSupportImageBuffer(const void* pMemLoc, int size);

   /**
    * Get the total number of pages in the image
    * \param  pMemLoc pointer to beginning of memory buffer
    * \param nSize buffer size in bytes
    * \return number of pages
    */
   int dlGetTotalPages(void* pMemLoc, int nSize);

   /**
    * Get the total number of pages in the image
    * \param  filename file name
    * \return number of pages
    */
   int dlGetTotalPages(const char* filename);
};//end class

static DLPPMImage * dlPpmProcessor = DLPPMImage::createImageReader();

#endif //_DLPPMIMAGE_H_
