#ifndef _DLPAGE_H_
#define _DLPAGE_H_

//#include "DLDocument.h"
//#include "DLZone.h"
#include "DLImage.h"
#include "DLTagList.h"
#include "DLConst.h"
#include "DLException.h"

#include <list>
#include <string>

class DLDocument;
class DLZone;

using namespace std;


/**
 * \brief Representation of a page within a DLDocument.
 *
 * In the DOCLIB document hierarchy, the DLPage class is below DLDocument
 * and above DLZone. Its main purpose is to accommodate information specific
 * to pages rather than whole documents. The structure of DLPage, however,
 * is similar to DLDocument. It also contains a page ID and a dynamic
 * tag list the developer can use to store self-defined attributes.
 * The main difference from the DLDocument class is a list of DLZone%s, which
 * are basically bounding boxes defining specific regions on a page. Following
 * the implementation of DLDocument, DLPage provides different functions for
 * appending, inserting, or deleting zones on a page. In addition, it provides
 * methods for accessing page information, e.g. width, height, or number of
 * zones. In particular, each page object provides a method for returning its
 * image as a DLImage object.
 *
 * Each DLPage object keeps track of two kinds of DLZone objects associated
 * with it -- zones on the DLPage, and zones that point to the DLPage (which
 * get created when copies of DLZones are made). Programmers can traverse
 * through the zones on a page using DLZonePtrIterator as below:
 *
 * \code
 * // ... pgPointer is a DLPage* to a page already populated with zones
 * DLPage::DLZonePtrIterator zoneIterator;
 * for (zoneIterator = pgPointer->begin(); zoneIterator != pgPointer->end(); zoneIterator++) {
 *   cout << (*zoneIterator)->dlGetZoneID() << endl;
 * }
 * \endcode
 *
 * Upon its destruction, the DLPage deletes all zones on the DLPage. Furthermore,
 * if there exist dangling zones that still point to the DLPage, the DLPage* on
 * these zones will be set to \c NULL, as the DLPage does not exist any more.
 *
 * It is advisable for programmers to use \c new operator to dynamically allocate memory
 * space when constructing DLPage%s and DLZone%s. The object destructors of DLDocument, 
 * DLPage, and DLZone will release the allocated memory space of associated objects
 * (i.e. all objects below the current object in the DLDocument hierarchy.) However, 
 * if you use the \c new operator to dynamically create an stand-alone object, you will need
 * to use the \c delete operator to reclaim the memory space it occupies.
 */

class DLPage
{
   friend class DLDocument;
   friend class DLZone;

   public:

   /**
    * Iterator which iterates over all the DLZone%*s in the page,
    * contained in the list pageZones.
    * Note that dereferencing this iterator gives a DLZone%*, which
    * must be dereferenced again to give an actual DLZone object reference.
    * (See the code example in the class introduction above.)
    */
   typedef std::list <DLZone*>::iterator DLZonePtrIterator;

   /**
    * Obtain an iterator pointing to the first of all the DLZone%*s in the page.
    * \return an iterator pointing to the first of all the DLZone*s in pageZones
    */
   DLZonePtrIterator begin() { return pageZones.begin(); }

   /**
    * Obtain an iterator that addresses the location succeeding the last DLZone%* in the page.
    * The end() iterator is typically used in a \c for loop for bounds checking
    * (see the example in the class introduction above). Warning: dereferencing the end()
    * iterator is undefined, and will likely cause your program to crash.
    * \return an iterator that addresses the location succeeding the last DLZone%* in pageZones
    */
   DLZonePtrIterator end() { return pageZones.end(); }
	
   /**
    * Check whether the given iterator is in the valid range
    * \return true if begin() <= \a zoneIter < end(), return false otherwise
    */
   bool dlIsZoneIteratorValid(DLZonePtrIterator zoneIter);

   /**
    * DLPage Constructor
    */
   DLPage();

   /**
    * DLPage Constructor
    * \param pageImage input image to initialize the protected DLImage member of the DLPage
    * \param pageID ID of the DLPage
    */
   DLPage(const DLImage &pageImage, const string &pageID);

   /**
    * DLPage Copy Constructor: all the DLZone%s on the page will also be copied over to the DLPage copy.
    * The DLPage copy points to the same DLDocument as the source DLPage. 
    * The parent DLDocument does not include this DLPage copy in the list of DLPage%s
    * that it owns; however, it maintains a reference to the new DLPage in its list
    * of backpointers so that it can notify the new DLPage to remove its parent
    * DLDocument reference when it is destroyed.
    * 
    * The DLPage copy has a zone hierarchy identical to
    * the source DLPage, in which each zone is a new object. As DLPage copy is NOT simply 
    * the default member-wise copy; it will not result in the accidental destruction of page zones
    * when the destructor of the DLPage copy is invoked.
    */
   DLPage(const DLPage &right);

   /**
    * Default DLPage Destructor
    */
   virtual ~DLPage();

   /**
    * Equality operator.
    * \return true if the two pages' ID and parent document are equal
    */
   bool operator == (const DLPage &right);

   /**
    * DLPage Copy Assignment operator:
    * all the DLZone%s on the page will be copied over to the DLPage copy.
    * The parent DLDocument does not include this DLPage copy in the list of DLPage%s
    * that it owns; however, it maintains a reference to the new DLPage in its list
    * of backpointers so that it can notify the new DLPage to remove its parent
    * DLDocument reference when it is destroyed.
    *
    * The DLPage copy has a zone hierarchy identical to
    * the source DLPage, in which each zone is a new object. As DLPage copy is NOT simply 
    * the default member-wise copy, it will not result in the accidental destruction of page zones
    * when the destructor of the DLPage copy is invoked
    */
   DLPage& operator=(const DLPage &right);


   /**
    * Get the unique ID of the page
    * \return the unique ID of the page
    */
   string dlGetPageID() const { return pageID; };

   /**
    * Set the ID string of the page
    * \param pgID new page ID
    */
   void dlSetPageID(const string &pgID) { pageID = pgID; };

   /**
    * Get page width
    * \return page width
    */
   int	dlGetPageWidth() const  { return pageImage.dlGetWidth(); };

   /**
    * Get page height
    * \return page height
    */
   int	dlGetPageHeight() const  { return pageImage.dlGetHeight(); };

   /**
    * Get the number of zones in the page
    * \return the number of zones in the page
    */
   int	dlGetNumZones() const  { return static_cast<int>(pageZones.size()); };

   /**
    * Check whether the current list of zones in the page is empty
    * \return true if page contains zones
    */
   bool dlHasZones() const { return (!pageZones.empty()); };

   /**
    * Append a new zone to the page at the end of the current list of zones.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by \a pageZone. Therefore, typically \a pageZone should have been allocated on the heap using
    * \c new. Taking the address of a DLZone stack object
    * for appending should be done with care,
    * since the object is automatically destroyed when it goes out of scope, potentially
    * causing a dangling pointer within the document. In this situation, 
    * consider using dlAppendZoneCopy() instead.
    * \param pageZone new zone to be appended
    */
   void dlAppendZone(DLZone *pageZone);

   /**
    * Append a copy of the new zone to the page at the end of the current list of zones.
    * \param pageZone new zone to be appended
    */
   void dlAppendZoneCopy(const DLZone &pageZone);

   /**
    * Append a list of new zones to the page at the end of the current list.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by the DLZone%s in \a zoneList.
    * \param zoneList list of new zones to be appended to the page at the end of the current list
    */
   void dlAppendZoneList(list <DLZone*> zoneList);

   /**
    * Insert a new zone to the page at the specified iterator position of the current list.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by \a pageZone.
    * See dlInsertZoneCopy() for inserting a stack DLZone object as a child zone.
    * \param zoneIter iterator position in the current zone list where the new zone is to be inserted
    * \param pageZone new DLZone to be inserted
    */
   void dlInsertZone(DLZonePtrIterator zoneIter, DLZone *pageZone);

   /**
    * Insert a new zone to the page at the specified position of the current list.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by \a pageZone.
    * See dlInsertZoneCopy() for inserting a stack DLZone object as a child zone.
    * \param cursorPosition cursor position in the current zone list where the new zone is to be inserted.
    *  \a cursorPosition must be between 0 (beginning of the list) and the number of zones
    *  currently on the list (end of the list)
    * \param pageZone new DLZone to be inserted
    */
   void dlInsertZone(int cursorPosition, DLZone *pageZone);

   /**
    * Insert a copy of the new zone to the page at the specified iterator position of the current list.
    * \param zoneIter iterator position in the current zone list where the new zone is to be inserted
    * \param pageZone DLZone to be copied and inserted
    */
   void dlInsertZoneCopy(DLZonePtrIterator zoneIter, const DLZone &pageZone);

   /**
    * Insert a copy of the new zone to the page at the specified position of the current list.
    * \param cursorPosition cursor position in the current zone list where the new zone is to be inserted.
    *  \a cursorPosition must be between 0 (beginning of the list) and the number of zones
    *  currently on the list (end of the list)
    * \param pageZone DLZone to be copied and inserted
    */
   void dlInsertZoneCopy(int cursorPosition, const DLZone &pageZone);

   /**
    * Insert a list of new zones into the page starting from the specified position in the current list.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by the DLZone%s in \a zoneList.
    * \param zoneIter iterator position in the current zone list where the new zones are to be inserted
    * \param zoneList list of new DLZone%s to be inserted at the specified cursor position in the current list
    */
   void dlInsertZoneList(DLZonePtrIterator zoneIter, list <DLZone*> zoneList);

   /**
    * Insert a list of new zones to the page starting from the specified position in the current list.
    * The DLPage object takes responsibility for freeing the memory occupied
    * by the DLZone%s in \a zoneList.
    * \param cursorPosition position in the current zone list where the new zones are to be inserted.
    *  \a cursorPosition must be between 0 (beginning of the list) and the number of zones
    *  currently on the list (end of the list)
    * \param zoneList list of new DLZone%s to be inserted at the specified cursor position in the current list
    */
   void dlInsertZoneList(int cursorPosition, list <DLZone*> zoneList);

   /**
    * Delete a specific zone from the page at the given iterator position
    * \param zoneIter DLZonePtrIterator of the zone to be deleted
    */
   void dlDeleteZone(DLZonePtrIterator zoneIter);

   /**
    * Delete a zone at a given position from the page
    * \param cursorPosition position of the zone to be removed from the zone list;
    * 0 indicates the first zone, 1 indicates the second zone, and so on.
    */
   void dlDeleteZone(int cursorPosition);

   /**
    * Delete a specific zone from the page
    * \param deletedZone zone to be deleted
    */
   void dlDeleteZone(DLZone *deletedZone);

   /**
    * Clear the list of zones in the page
    */
   void dlClearZones();

   /**
    * Get a copy of the page image
    * \return page image
    */
   DLImage dlGetImage() { return pageImage; };

   /**
    * Check whether the page image has been initialized
    * \return true if the page image has been initialized
    */
   bool dlCheckPageImageData();

   /// \name Tag List Functions
   //@{

   /**
    * Get the field value of a specified page tag
    * \param tagKey key of the specified page tag
    * \return the field value of the specified page tag 
    */
   string dlGetTag(string tagKey) const;

   DLTagList dlGetAllTags() const { return pageTags; };
	
   /**
    * Set a page tag. If a tag with the specified key already exists
    * in this page, it will not be overwritten unless the optional parameter
    * \a overwriteEnabled is set to \c true.  Will throw a DLException if \a overwriteEnabled is 
    * set to \c false and \a tagKey exists.
    * \param tagKey key of the document tag
    * \param tagValue value of the document tag
    * \param overwriteEnabled option for overwriting existing field (default is \c false)
    * \throw DL_Exception DL_UNKNOWN_TAG_EXCEPTION
    */
   void dlSetTag(string tagKey, string tagValue, bool overwriteEnabled = false);

   /**
    * Remove the tag with the specified key from the list of page tags.
    * \param tagKey key of the page tag
    */
   void dlDeleteTag(string tagKey);

   /**
    * Erase all the existing page tags
    */
   void dlClearTags() { pageTags.tagMap.clear(); };

   /**
    * Check whether a document tag with the given key exists
    * \param tagKey key of the document tag
    * \return true if <i><b>a</b> tag exists</i> with that key
    */
   bool dlIsTagSet(string tagKey) const;

   /**
    * Check whether the whole page tag list is empty
    * \return true if the page tag list is empty
    */
   bool dlIsTagListEmpty()  const { return pageTags.tagMap.empty(); };

   /**
    * Get an iterator pointing to the location of \c tagKey in the map
    * \return iterator
    */
   DLTagList::iterator dlFindTag ( string tagKey ) { return pageTags.tagMap.find(tagKey); }
   
   //@}

   protected:

   string pageID;

   DLImage pageImage;

   DLDocument* pageDocument;

   // list of attributes
   DLTagList pageTags;

   // list of zones
   list <DLZone*> pageZones;

   // list of zones whose DLPage *zonePage point to this page
   // this enables the DLPage to keep track of its associated DLZones when its destructor is invoked
   list <DLZone*> zoneBackPointers;

   /**
    * Create a copy of the current DLPage object and return the pointer of this new copy as precisely DLPage* type
    * This is a utility member function solely used by the copy constructor and assignment operator
    * to ensure that the correct derived type of DLPages on the document is recognized when a copy
    * of the document is created.
    * In order to fully support derived classes of DLPage, each derived class of DLPage must have
    * a similar member function that returns a copy of the current derived object in its correct derived type.
    */
   DLPage* dlClone();

   /**
    * Set the DLDocument* of the current page without adding the page to the document
    * if the DLDocument* of the current page has been set to another existing document, the page
    * will be removed from the pageBackPointers of that document before being added to the new document
    */
   void dlSetDocumentPointer(DLDocument *pgDocument);

}; //end class

#endif //_DLPAGE_H_
