#ifndef _DLRECTSHAPE_H_
#define _DLRECTSHAPE_H_

#include "DLShape.h"
#include "DLZone.h"

/**
 * \brief An object used to represent a rectangle
 * 
 * DLRectShape is a shape object that represents a rectangle
 * by a point (<i>x</i>,<i>y</i>) representing its
 * origin, or upper-left corner, and its width and height.  
 * The DLRectShape object can also be used to
 * draw a rectangle onto a DLImage with a desired 
 * line color and fill color.
 * 
 * Typical usage:
 * \code 
 * // load an image
 * DLImage myImage("myfile.tif");
 * // define the point
 * DLRectShape rect(1,1, 20,40);
 * //Set the line and fill color to DL_BLANCHEDALMOND
 * rect.dlSetLineColor(DLColor(DL_BLANCHEDALMOND));
 * rect.dlSetFillColor(DLColor(DL_BLANCHEDALMOND));
 * //Draws the rectangle onto a DLImage
 * rect.dlDrawShape(&myImage);
 * 
 * \endcode
 */
class DLRectShape : public DLShape
{
   public:
	
   /**
    * Default DLRectShape constructor
    */
   DLRectShape( ): DLShape()   { };
   /**
    *  DLRectShape Copy  Constructor
    * \param orig DLRectShape to be copied
    */
   DLRectShape(const DLRectShape & orig);

   /**
    *  Assignment operator
    * \param right DLRectShape to be copied
    */
   const DLRectShape & operator=(const DLRectShape & right);

   /**
    * DLRectShape constructor.  It takes a point on 
    * the image for the rectangle origin, and width and height.  
    * \param x1 column, or <i>x</i>-coordinate of the point
    * \param y1 row, or <i>y</i>-coordinate of the point
    * \param w1 width of rectangle
    * \param h1 height of rectangle
    */
   DLRectShape(int x1, int y1, int w1, int h1) : DLShape()
      { x = x1; y = y1;  w = w1;  h = h1; };

   /**
    * DLRectShape constructor.  It takes a DLZone and will construct
    * a drawable Rectangle shape based on the details of the DLZone
    * \param z DLZone containing Recatangle Shape Information
    */
   DLRectShape(const DLZone & z) : DLShape()
      { 
         x = z.dlGetZoneOrigin().x; 
         y = z.dlGetZoneOrigin().y;  
         w = z.dlGetZoneWidth();    
         h = z.dlGetZoneHeight();    
      };

   /**
    * DLRectShape constructor.  It takes two DLPoints detailing the upperleft
    * and lower right points of the Rectangle to be created
    * \param upperleft DLPoint containing upperleft point of the DLRectShape
    * \param lowerright DLPoint containing lowerright point of the DLRectShape
    */
   DLRectShape(const DLPoint & upperleft, const DLPoint & lowerright) : DLShape()
      { 
         x = upperleft.x; 
         y = upperleft.y;  
         w = lowerright.x-upperleft.x;    
         h = lowerright.y-upperleft.y;    
      };

   /**
    * Default DLRectShape Destructor
    */
   virtual ~DLRectShape() {};

   /**
    * Draws the rectangle shape onto an image
    * \param image image to draw onto, passed by pointer (see above code example)
    */
   void dlDrawShape(DLImage* image);

    
   /**
    * Get X 
    * \return column, or <i>x</i>-coordinate of the rectangle's origin 
    */
   int dlGetX()const{return x;};
   /**
    * Get Y
    * \return row, or <i>y</i>-coordinate of the rectangle's origin 
    */
   int dlGetY()const{return y;};
   /**
    * Get Width
    * \return width of the rectangle 
    */
   int dlGetWidth()const {return w;} ;
   /**
    * Get Height
    * \return height of the rectangle
    */
   int dlGetHeight()const {return h;};

   /**
    * Set rectangle's origin, or upper-left point.
    * \param xPoint column, or <i>x</i>-coordinate of the rectangle's origin 
    */
   void dlSetX(int xPoint){x = xPoint;};
   /**
    * Set rectangle's origin, or upper-left point.
    * \param yPoint row, or <i>y</i>-coordinate of the rectangle's origin 
    */
   void dlSetY(int yPoint){y = yPoint;};
   /**
    * Set height
    * \param height new height of rectangle 
    */
   void dlSetHeight(int height){h = height;};
   /**
    * Set width
    * \param width new width of rectangle 
    */
   void dlSetWidth(int width){w = width;};

   protected:
   /// position of rectangle, X coordinate
   int x;
   /// position of rectangle, Y coordinate
   int y;
   /// width of rectangle
   int w;
   /// height of rectangle
   int h;			
	
};  

#endif
