#ifndef _DLSHAPE_H_
#define _DLSHAPE_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLColor.h"
#include "DLLogger.h"


/**
 * Line types: dashed or solid. 
 * This is used to indicate the type of line to use
 * when drawing a shape. 
 * \deprecated Use DLShape::DLLineStyle instead
 * <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
 */
enum DLLineStyle {DL_SOLID, DL_DASH};

/**
 * \brief Abstract base class for all shape types.
 *
 * It provides the common functions/interfaces that all derived 
 * shape objects must implement.  
 * 
 */
class DLShape
{
   public:

	/**
	 * Line types: dashed or solid. 
	 * This is used to indicate the type of line to use
	 * when drawing a shape. 
	 */
	 enum DLLineStyle {DL_SOLID, DL_DASH};

   /// \name Constructors, Destructors & Operators
   //@{

   /**
    * Default DLShape Constructor
    */
   DLShape() ;

   /**
    *  DLShape Copy  Constructor
    *  \param orig DLShape to be copied
    */
   DLShape(const DLShape & orig);

   /**
    * Default DLShape Destructor
    */
   virtual ~DLShape() {};

   /**
    *  Assignment operator
    * \param right DLShape to be copied
    */
   const DLShape & operator=(const DLShape & right);

   //@}

   /// \name Set Methods
   //@{

   /**
    * Set the line color for drawing a shape onto a DLImage.
    * \param color line color
    */
   void dlSetLineColor(const DLColor& color);

   /**
    *  Set the thickness of the line used when drawing the shape onto a DLImage.
    * \param thickness thickness of the line.  Must be > 0
	* \deprecated Use DLShape::dlSetLineThickness instead
    * <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   void dlSetLineStyle(int thickness)	
      {
         dlLogger.dlLog(DL_CRITICAL, "DLShape::dlSetLineStyle() has been replaced with dlSetLineThickness()");
         lineThickness = 1; 
      };  

   void dlSetLineThickness(int thickness) {lineThickness = thickness;};
   /**
    * Set whether the line will be drawn solid or dashed.
    * \param DashStyle line style
    */
   void dlSetDashStyle(DLShape::DLLineStyle DashStyle) {lineStyle = DashStyle; }; 

   /**
    * Set the length of the dashes and spaces when line is dashed.  Default is 5
    * \param length
    */
   void dlSetDashLength( int length ) { dashLength = length; };


   /**
    *  Set the fill color for the shape object. 
    *  The fill color is used to fill in the interior of the shape when drawn onto a DLImage.
    * \param color fill color
    */
   void dlSetFillColor(const DLColor& color) ;

   /** Tell the shape to not fill itself.  Turns off \c fillShape
    */

   void dlSetFillOff() { fillShape = false;} ;

   //@}
   /// \name Get Methods
   //@{

   /**
    *  Get line color	
    * \return Line color
    */
   DLColor dlGetLineColor( void ) const { return lineColor;} ; 

   /**
    *  Get line thickness	
    * \return line thickness
	* \deprecated Use DLShape::dlGetLineThickness instead
    * <br>This is in <b>Phase One</b> of the <i>Documented Deprecation Process</i>
    */
   int dlGetLineStyle( void ) const {
   dlLogger.dlLog(DL_CRITICAL, "DLShape::dlGetLineStyle() has been replaced with dlGetLineThickness()");
   return lineThickness;
   };

   /**
    *  Get line thickness	
    * \return line thickness
    */
   int dlGetLineThickness( void ) const { return lineThickness;};
   /**
    *  Get dash style	
    * \return DLLineStyle
    */
   DLShape::DLLineStyle dlGetDashStyle( void ) { return lineStyle;};

   /**
    *  Get dash length	
    * \return int length
    */
   int dlGetDashLength ( void ) { return dashLength; };

   /**
    *  Get fill color
    * \return DLColor
    */
   DLColor dlGetFillColor( void ) { return fillColor;};

	//@}

   /**
    * Draws the shape onto an image. Example:
    * \code
    * DLImage i("test.tif");
    * DLRectShape box(5,5,30,50);
    * box.dlDrawShape(&i);
    * \endcode
    * \param image DLImage to draw onto
    */
   virtual void	dlDrawShape(DLImage * image) = 0;

   protected:
   ///line color
   DLColor	lineColor;

   /// color used to fill shape when drawing 
   DLColor fillColor;

   ///fill flag
   //bool fillFlag;

	
   /// line style: dashed or solid
   DLShape::DLLineStyle	lineStyle;

   /// if dashed, length of dash
   int dashLength;

   /// the thickness of the drawing line
   int	lineThickness;
   /** 
    * A flag to indicate
    * to the draw function whether to fill in the shape with 
    * the color indicated by fillColor when drawing the shape.
    */
   bool fillShape;
};




#endif
