#ifndef DLUTILS_H
#define DLUTILS_H

#include "DLUtils.h"

/**
 * System byte order.
 */
enum DLEndian {
   DL_LE, ///< Little Endian
   DL_BE  ///< Big Endian
};

/**
 * \brief General utility class.
 *
 * DLUtils class is a utility class to 
 * store commonly used functions. All functions
 * in the DLUtils class are static and can be called
 * without first creating an instance of DLUtils. 
 *
 * Usage:
 * \code 
 * double doubleInt = 1.999234;
 * int myInt = DLUtils::roundInt(doubleInt);
 * \endcode
 * 
 */
class DLUtils
{
   public:
   /** \cond **/
   /**
    * constructor   
    */
   DLUtils(void);

   /**
    * Destructor
    */
   ~DLUtils(void);

   /** \endcond **/
    
   /**
    * Rounds a double value to the nearest integer.
    *
    * The roundInt function is useful when casting the result of 
    * a floating point operation (multiplication, division, and etc.) to an integer.
    * The result of floating operation are OS dependent because every OS  
    * may have a different floating point precision.  Depending on 
    * closeness of the result to the precision boundary, the result
    * may get rounded up to the next higher integer value for certain OS/Compiler.
    * If the traditional cast <tt>(int)(2/3)</tt> is used, the result for different 
    * OSs may have be different by 1.
    *
    * \param val value to be rounded
    * \return rounded integer value
    */
   static int roundInt(double val); 

   /**
    * Rounds the double value to the
    * precision (number of decimal places) indicated.
    * Example:
    * \code
    * double x = 3.14159;
    * double y = DLUtils::roundDouble(x,4);
    * \endcode
    * yields <tt>y = 3.1416</tt>.
    * \param value value to be rounded
    * \param precision round value to this precision
    * \return rounded double vaue to the indicated precision
    */
   static double roundDouble(double value, int precision);

   /**
    * reverse the byte order of an integer
    * \code
    * [1][2][3][4] -> [4][3][2][1]
    * \endcode
    *
    * \param iVal value to reverse byte order
    */
   static void reverseInt(int &iVal);
   /**
    * reverse the byte order of a short
    * \code
    * [1][2] -> [2][1]
    * \endcode
    * \param sVal value to reverse byte order
    */
   static void reverseShort(short &sVal);

   /**
    * Determines the endianness (byte order) of the
    * current system.
    * 
    * It declares a two-byte \c short \c int value 0x0001.
    * If the first byte in memory is 0x01, the system is little endian.
    * If the first byte in memory is 0x00, the system is big endian.
    *
    * \return endian type
    */
   static DLEndian testByteOrder()
      {
         short int word = 0x001;  // two byte
         char *byte = (char*) &word; // one bye variable gets first byte
         return (byte[0] ? DL_LE : DL_BE);  // if it's one, we have a little endian system
      };

   /***************************************************************************
    *  Given a filename with or without concatenated path, creates the root filename
    *  For example:  Input "out.tif" yields output "out"
    *  \param longFile "a filename"
    *  \return A filename if successful, NULL otherwise
    ***************************************************************************/
   static char * getRootName (char *longFile);

   /***************************************************************************
    *  Given a filename with or without concatenated path, creates the suffix
    *  For example:  Input "out.tif" yields output "tif"
    *  \param longFile "a filename"
    *  \return A filename if successful, NULL otherwise
    ***************************************************************************/
   static char * getSuffix (char *longFile);
};


#endif // DLUTILS_H
