#ifndef _DLELLIPSEDETECT_H_
#define _DLELLIPSEDETECT_H_

/**
* The DLEllipseDetect class detects elliptic objects within input
* images by formulating and characterizing connected edges using pair-wise second-order 
* representation. This technique exploits the orientation information of pair of edges
* in order to infer the location and approximated area of the detected object using an effcient
* Hough transform voting scheme.
* 
* It generates a list of candidate objects of DLEllipseCandidate type, which store
* their information including zone region, confidence values.
*/

#include "DLImage.h"
#include "DLZone.h"
#include "DLComponent.h"

#include <list>
#include <stdio.h>
#include <math.h>

using namespace std;

#define RESIZE_FILTER_WIDTH 3
#define AXIS_RATIO_MAX 3.0
#define BIG_LAM 1000.0
#define SMALL_LAM exp(-3.0)
#define MAX_AREA 1.0
#define MAX_DATARUN_LENGTH 50
#define PERCENGAGE_SCORES_THRESHOLD 0.10
#define MIN_SPATIAL_DENSITY 0.20
#define MIN_SCORE_THRESHOLD 8000

// sets of characteristics used in filtering out unrelevant connected components from edges
#define MIN_CC_AREA 10
#define MAX_CC_ROW_LENGTH 500
#define MAX_CC_COL_LENGTH 500
#define MIN_CC_ROW_LENGTH 20
#define MIN_CC_COL_LENGTH 20
#define MIN_ANGLE_DIFF 0.393
#define MIN_MISSING_PIXELS 8
#define MAX_MISSING_PIXELS 100

#define max(x,y) ( ((x) > (y)) ? (x) : (y) )
#define min(x,y) ( ((x) < (y)) ? (x) : (y) )

// struct for storing an ellipse candidate with their corresponding confidence values
struct DLEllipseCandidate
{
	//DLImage ellipseRegionImage;
	DLZone ellipseRegion;
	DLZone ccRegion;
	string sourceImageName;
	bool tested;
	int scores;
	double confidenceValue;
	string type;
};

// struct for bookkeeping value of 3-dimensional accumulator bins
struct DL3DBin
{
	int indexI;
	int indexJ;
	int indexK;
	int scores;
};

class DLEllipseDetect
{
public:

	/**
	* Default DLEllipseDetect Constructor
	*/
	DLEllipseDetect() { initialized=0; };

	/**
	* DLEllipseDetect Constructor
	*
	* Detects elliptic and complex shape objects from input images and return a list detected ellipses 
	* with their corresponding confidence values.
	*
	* \param image DLImage input image
	* \param mag_thres DL_BYTE threshold value used for evaluating the edge magnitude image.
	* Edges with magnitude weaker than this value are ignored. Its valid range is [1, 255]
	* \param image_shrink_ratio double the image shrink ratio (>=1)
	*
	* A priori information of the wanted elliptic objects can be used 
	* to effectively improve the detection accuracy and further constraint
	* the search space for faster processing. If currently there is no
	* available a priori information on the area of the searched elliptic 
	* objects, you can simply input area_min_pixel=0, area_max_pixel=rows*cols.
	* In this case, there are no constraints posed on the range of ellipse area.
	*
	* \param area_min_pixel int the minimum area of the elliptic objects in pixel square in original input image
	* \param area_max_pixel int the maximum area of the elliptic objects in pixel square in original input image
	* \param axis_ratio_max float the maximum possible ratio of the lengths of 
	* the major axis to the minor axis
	*
	* The following four parameters specify the region of the search within the image
	* with the full range of each scaled to [0, 1.0]. x_left=0, x_right=1.0, y_left=0, 
	* y_right=1.0 specify that the search is to be performed in the entire image.
	* \param y_center0 float starting position in row, valid range [0, 1.0]
	* \param y_center1 float ending position in row, valid range [0, 1.0]
	* \param x_center0 float starting position in column, valid range [0, 1.0]
	* \param x_center1 float ending position in column, valid range [0, 1.0]
	*
	* The following three parameters specify the number of accumulator bins 
	* for voting in each dimension, i.e. the center of the ellipse in row direction,
	* the center of the ellipse in column direction, and the area of the ellipse.
	* \param ny_center int the number of distinctive accumulator bins for ellipse center in row direction
	* \param nx_center int the number of distinctive accumulator bins for ellipse center in column direction
	* \param narea int the number of distinctive accumulator bins for ellipse area
	*/
	DLEllipseDetect(DLImage &image, DL_BYTE mag_thres, double image_shrink_ratio,
					int area_min_pixel, int area_max_pixel, float axis_ratio_max,
					float y_center0=0, float y_center1=1.0, float x_center0=0, float x_center1=1.0,
					int ny_center=100, int nx_center=100, int narea=100);

	/**
	* Default DLEllipseDetect Destructor
	*/
	virtual ~DLEllipseDetect() {};

	/**
	* Draw colored bounding boxes around the specified ellipse candidates
	* \param outputImage DLImage output color image
	* \param bins DL3DBin* to 3-dimensional accumulator bins
	* \param binIndex int the index of the 3-dimensional accumulator bin
	* \param markerColor DLColorType the color of the plotted bounding box
	*/
	void dlDrawEllipseRegion(DLImage &outputImage, DL3DBin *bins, int binIndex, enum DLColorMapValue markerColor=DL_RED1);

	/**
	* Return the list of ellipse candidates
	* \return the list of ellipse candidates in DLEllipseCandidate type
	*/
	list <DLEllipseCandidate> dlGetDetectedEllipses() { return ellipseList; };

private:

	bool initialized;

	// essential information of the processed image which is possibly rescaled from the original image
	int rows, cols;

	// image shrink ratio (>= 1) gives correpondence between the processed rescaled image and its original version
	double resize_ratio;

	list <DLEllipseCandidate> ellipseList;

	DL3DBin *bins;

	// the number of distinctive accumulator bins for each dimension
	int ny_center, nx_center, narea;
    
	/**
	* Populate the 3-dimensional accumulator bins and vote for the parameter representation
	* in the feature space, i.e. ellipse center in row direction, ellipse center in column 
	* direction, and ellipse area
	*
	* \param magImage DLImage grayscale edge magnitude image
	* \param oriImage DLImage grayscale edge orientation image
	* \param mag_thres DL_BYTE threshold value used for evaluating the edge magnitude image.
	* Edges with magnitude weaker than this value are ignored.
	*
	* A priori information of the wanted elliptic objects can be used 
	* to effectively improve the detection accuracy and further constraint
	* the search space for faster processing. If currently there is no
	* available a priori information on the area of the searched elliptic 
	* objects, you can simply input area_min_pixel=0, area_max_pixel=rows*cols.
	* In this case, there are no constraints posed on the range of ellipse area.
	* \param area_min_pixel int the minimum area of the elliptic objects in pixel square in original input image
	* \param area_max_pixel int the maximum area of the elliptic objects in pixel square in original input image
	* \param axis_ratio_max float the maximum possible ratio of the lengths of 
	* the major axis to the minor axis
	*
	* The following four parameters specify the region of the search within the image
	* with the full range of each scaled to [0, 1.0]. x_left=0, x_right=1.0, y_left=0, 
	* y_right=1.0 specify that the search is to be performed in the entire image.
	* \param y_center0 float starting position in row, valid range [0, 1.0]
	* \param y_center1 float ending position in row, valid range [0, 1.0]
	* \param x_center0 float starting position in column, valid range [0, 1.0]
	* \param x_center1 float ending position in column, valid range [0, 1.0]
	*
	* \param score int* to the int array that stores the score of the accumulator bins
	*/	
	void dlPopulateBins(DLImage &magImage, DLImage &oriImage, DL_BYTE mag_thres, int area_min_pixel, int area_max_pixel, float axis_ratio_max,
						float y_center0, float y_center1, float x_center0, float x_center1, int *score);
	
	/**
	* Calculate the parameter set (a,b,c,f,g,h) at the given lambda value for the two edge points at [x1, y1] and [x2, y2]
	* the gradient direction at points [x1, y1] and [x2, y2] are -p1/q1 and -p2/q2, respectively
	*/
	void dlComputeParameters(float x1,float y1,float p1,float q1, float x2,float y2,float p2,float q2,
							float lambda,float *a,float *b,float *c,float *f,float *g,float *h);

	/**
	* Calculate the upper bound of parameter lambda for the two edge points at [x1, y1] and [x2, y2]
	* the gradient direction at points [x1, y1] and [x2, y2] are -p1/q1 and -p2/q2, respectively
	* \return float the upper bound of parameter lambda for the two edge points at [x1, y1] and [x2, y2]
	*/
	float dlFindLambdaRange(float x1,float y1,float p1,float q1,float x2,float y2,float p2,float q2);

	/**
	* Calculate the center position [x0, y0] of the ellipse with the corresponding parameter set of (a,b,c,f,g,h)
	* \return the center position [x0, y0] of the ellipse with the corresponding parameter set of (a,b,c,f,g,h)
	*/
	void dlComputeCenter(float *x0,float *y0,float a,float b,float c,float f,float g,float h);

	/**
	* Calculate the area of the ellipse with corresponding parameter set (a,b,c,f,g,h)
	* \return float the area of the ellipse with corresponding parameter set (a,b,c,f,g,h)
	*/
	float dlComputeArea(float a,float b,float c,float f,float g,float h);

	/**
	* Compute the eigenvalues lam1 and lam2 of the 2 X 2 matrix [a   h; h   b] and place them in the order lam1 > lam2
	*/
	void dlComputeEigenValue(float *lam1,float *lam2,float a,float b,float h);

	/**
	* Calculate the angle of the main ellipse axis wrt x axis
	* \return the angle of the main ellipse axis within range (-PI/2, PI/2]
	*/
	float dlComputeOrientation(float a,float b,float c,float f,float g,float h);

	/**
	* Calculate the ratio between the major axis and the minor axis
	* \return float the ratio between the major axis and the minor axis
	*/
	float dlComputeMajorMinorRatio(float a,float b,float c,float f,float g,float h);

	/**
	* Calculate the ratio between the major axis and the minor axis and check whether it is in the valid range
	* \return bool true is in the range, false otherwise
	*/
	bool dlCheckMajorMinorRatio(float a,float b,float c,float f,float g,float h,float axis_ratio_max);

	void dlRemoveLongJunctions(DLImage &magImage, DL_BYTE mag_thres);

	/**
	* Break elliptic connected component at the junction points where its overlaps with background content
	*/
	void dlRemoveShortJunctions(DLImage &magImage, DL_BYTE mag_thres);

	/**
	* Remove a vertically connnected strip of edge pixels from the edge intensity image
	*/
	void dlRemoveVerticalStrip(DL_BYTE* magData, int juncRow, int juncCol, DL_BYTE mag_thres);

	/**
	* Remove a horizontally connnected strip of edge pixels from the edge intensity image
	*/
	void dlRemoveHorizontalStrip(DL_BYTE* magData, int juncRow, int juncCol, DL_BYTE mag_thres);

	/**
	* Filter large connected components from the edge intensity image
	*/
	void dlFilterLargeCC(DLImage &magImage, DL_BYTE edgeThresold);

	/**
	* Remove the specified connected component from the edge intensity image
	*/
	void dlRemoveCC(DLImage &magImage, DLComponent &largeC);

	/**
	* Verify the detected region
	*/
	void dlVerifyEllipseCandidates(DLImage &image);

	/**
	* Locate and further refine the boundary of the detected region
	*/
	void dlLocateCCBoundary(DLImage &magImage, DLEllipseCandidate &detectedEllipse, DL_BYTE edge_thres);


};//end class

#endif //_DLELLIPSEDETECT_H_
